#!/bin/bash
#
# Copyright (C) 2022 Actiphy, Inc.
#   All rights reserved.
#
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Actiphy, Inc.
# The copyright notice above does not evidence any actual or intended
# publication of such source code.
#
# AIP-packages-tool.sh: AIP packages install tool.
# Created by Yoshinobu Ito
# File:  AIP-packages-tool.sh
#
# Date:  11/17/22
#
# Hist:
#	ver 0.2.45
#
# Qualified Platforms:
#	Redhat 6.x/7.x/8.x/9.x
#	CentOS 6.x/7.x/8.x
#	AlmaLinux 8.x/9.x
#	Oracle Linux 6.x/7.x/8.x/9.x(UEK supported)
#	Rocky Linux 8.x
#	OpenSuse 15.x
#	SLES 15.x
#	Ubuntu 18/20/22LTS
#	Debian 9/10/11
#
# Qualified Architecture:
#	x86_64
#
# Qualified File System:
#	ext2,3,4 xfs
#
# Requirements Product:
#	AIP Linux version 2018 Update7 or later.
#
saved_lang="$LANG"
export LANG=C
ScVer="0.2.45"

tty=$(readlink /proc/$$/fd/2)

#FIFO=/tmp/$$.fifo
LOG=/tmp/AIP-packages-tool.log
#mkfifo ${FIFO}
#trap '/bin/rm -f ${FIFO}' exit
#tee ${LOG} < ${FIFO} &
#exec > ${FIFO} 2>&1
exec > >(tee ${LOG}) 2>&1
dir=$(dirname $(readlink -f $0))

color_off="\e[m"
color_red="\e[31m"
color_blue="\e[34m"
color_magenta="\e[35m"
color_blue_back="\e[44m"

if [ `id -u` -ne 0 ]; then
	echo -e "${color_red} Only root user can install this product.${color_off}"
	exit
fi

ker_list="
8.5.2111 8.5 4.18.0-348
8.4.2105 8.4 4.18.0-305
8.3.2011 8.3 4.18.0-240
8.2.2004 8.2 4.18.0-193
8.1.1911 8.1 4.18.0-147
         8.0 4.18.0-80
7.9.2009 7.9 3.10.0-1160
7.8.2003 7.8 3.10.0-1127
7.7.1908 7.7 3.10.0-1062
7.6.1810 7.6 3.10.0-957
7.5.1804 7.5 3.10.0-862
7.4.1708 7.4 3.10.0-693
7.3.1611 7.3 3.10.0-514
7.2.1511 7.2 3.10.0-327
7.1.1503 7.1 3.10.0-229
7.0.1406 7.0 3.10.0-123
6.10 2.6.32-754
6.9 2.6.32-696
6.8 2.6.32-642
6.7 2.6.32-573
6.6 2.6.32-504
6.5 2.6.32-431
6.4 2.6.32-358
6.3 2.6.32-279
6.2 2.6.32-220
6.1 2.6.32-131.0.15
6.0 2.6.32-71"

echo System checking....

pre_suse_repo() 
{
echo "=== Internet connection failed. Please insert the OS DVD."
echo -n "Input DVD mount path [e.g.: /run/media/root/SLE-15-SP2-Full-x86_64209.21.001] :"
read dvd_path
if [ ! -e $dvd_path ]; then
	echo "Path not found."
fi

repo_arry=()
repo_arryE=()

repodir="/etc/zypp/repos.d"

repos_count=`ls $dvd_path | grep -c Module`
if [ $repos_count -ne 0 ]; then
	repos=`ls $dvd_path | grep Module`
	repo_arry+=($repos)
	BK_IFS=$IFS
	IFS=$'\n'
	repo_count=`echo "${repo_arry[*]}" | grep -e Basesystem -e Desktop-Applications -e Development-Tools | wc -l`
	if [ $repo_count -eq 3 ]; then
		repo_arryE+=(`echo "${repo_arry[*]}" | grep -e Basesystem -e Desktop-Applications -e Development-Tools`)
		for i in ${repo_arryE[*]}
		do
			zypper addrepo -G -n DVD $dvd_path DVD-$i
			if [ -f $repodir/DVD-$i.repo ]; then
				sed -i "$ a path=/$i" $repodir/DVD-$i.repo
			fi
		done
	fi
	zypper ref 
else
	repos=`ls $dvd_path | grep repodata`
	repo_arry+=($repos)
	BK_IFS=$IFS
	IFS=$'\n'
	repo_count=`echo "${repo_arry[*]}" | grep -e repodata | wc -l`
	if [ $repo_count -eq 1 ]; then
		zypper addrepo -G -n DVD $dvd_path DVD
		if [ -f $repodir/DVD.repo ]; then
			sed -i "$ a path=/" $repodir/DVD.repo
		fi
	fi
	zypper ref 
fi
IFS=$BK_IFS
}

pre_debian_repo(){

apt_src="/etc/apt/sources.list"
sed -i '/deb cdrom/d' $apt_src

ping -c 1 www.google.com >/dev/null 2>&1
if [ $? -eq 0 ]; then
	sed -i 's/^deb cdrom/\#deb cdrom/' $apt_src
	apt-add-repository -y main >/dev/null
	apt-add-repository -y non-free >/dev/null
	apt-add-repository -y contrib >/dev/null
	echo "Run apt update....."
	apt update -y >/dev/null 2>&1
	recreate_cdrom=0
else
	sed -i 's/^deb http/\#deb http/' $apt_src
	exit_cdrom_repo=`cat $apt_src | grep "deb cdrom" | wc -l`
	enable_cdrom=`cat $apt_src | grep "deb cdrom" | grep -v "#" | wc -l`
	if [ $exit_cdrom_repo -eq 0 ]; then
		echo "=== Internet connection failed. Please insert the OS DVD."
		echo -n "Input DVD mount path [e.g.: /media/cdrom0] :"
		read dvd_path
		if [ ! -e $dvd_path ]; then
			echo "Path not found."
		fi
		apt-cdrom add
		apt-cdrom ident
		echo "Run apt update....."
		apt update -y >/dev/null 2>&1
		recreate_cdrom=1
	elif [ $exit_cdrom_repo -ne 0 ] || [ $enable_cdrom -ne 0 ]; then
		echo "Enable cdrom repository."
		sed -i 's/^# deb cdrom/\deb cdrom/' $apt_src
		apt-cdrom ident
		echo "Run apt update....."
		apt update -y >/dev/null 2>&1
		recreate_cdrom=1
	fi
fi

}

#Check system requirment
Check_Dist(){

if [ -f /etc/os-release ]; then
	DIST=`cat /etc/os-release | grep -e "^NAME" | sed -e 's/^NAME=//' -e 's/^\"//' -e 's/$*"//' | awk -F " " '{print $1}'`
	OSVER=`cat /etc/os-release | grep -e "^VERSION_ID" | sed -e 's/^VERSION_ID="//' -e 's/^VERSION_ID=//' -e 's/$*"//' -e s'/\.[0-9]*//' | sed -e 's/ *$//'`
else 
	DIST=`cat /etc/system-release | awk -F " " '{print $1}'`
	OSVER=`cat /etc/system-release | grep -oE '([0-9]\.[0-9]*)' | sed -e 's/\.[0-9]*//'| sed -e 's/ *$//'`
fi

case "$DIST" in
	"Red") OSDIST="RHEL" ;;
	"CentOS") OSDIST="CentOS" ;;
	"Asianux") OSDIST="Asianux" ;;
	"Oracle") OSDIST="Oracle" ;;
	"AlmaLinux") OSDIST="AlmaLinux" ;;
	"MIRACLE") OSDIST="MIRACLE" ;;
	"Rocky") OSDIST="Rocky" ;;
	"openSUSE") OSDIST="openSUSE" ;;
	"SLES") OSDIST="SLES" ;;
	"SLED") OSDIST="SLED" ;;
	"Debian") OSDIST="Debian" ;;
	"Ubuntu") OSDIST="Ubuntu" ;;
	"Amazon") OSDIST="Amazon" ;;
	*) echo "This Distribution does not support." 
	   exit 1
esac

if [ "$OSDIST" = "Debian" -a $OSVER -eq 9 ]; then
	OSVER="94"
fi

if [ "$OSDIST" = "SLES" ] || [ "$OSDIST" = "SLED" ]; then
	suse_min_ver=`cat /etc/os-release | grep -e "^VERSION_ID" | sed -e 's/^VERSION_ID="//' -e 's/^VERSION_ID=//' -e 's/$*"//'`
fi
if [ "$OSDIST" = "RHEL" ] || [ "$OSDIST" = "CentOS" ] || [ "$OSDIST" = "Oracle" ] || [ "$OSDIST" = "AlmaLinux" ] || [ "$OSDIST" = "MIRACLE" ] || [ "$OSDIST" = "Rocky" ]; then
	min_ver=`cat /etc/system-release | grep -oE "[0-9]\.[0-9]*"`
fi
if  [ "$OSDIST" = "Asianux" ]; then
	min_ver=`cat /etc/system-release | grep -oE "SP[0-9]" | sed 's/SP//'`
fi
if [ "$OSDIST" = "Ubuntu" ]; then
	ubuntu_lts=`cat /etc/os-release | grep PRETTY_NAME | grep LTS | wc -l`
	if [ $ubuntu_lts -eq 0 ]; then
		name=`cat /etc/os-release | grep PRETTY_NAME | sed 's/PRETTY_NAME="//' | sed 's/\"//'`
		echo "OS version [$name]."
		echo "AIP supported LTS version."
		exit 1
	fi
fi

# ./scripts/pahole-flags.sh: line 7: return: can only `return' from a function or sourced script
# https://lore.kernel.org/lkml/CAHk-=wgdE6=ob5nF60GvRYAG24MKaJBGJf3jPufMe1k_UPBQTA@mail.gmail.com/
# pahole-flags.sh bugs
if [ "$OSDIST" = "Ubuntu" -a $OSVER -eq 22 ]; then
	pahole=/usr/src/linux-headers-$(uname -r)/scripts/pahole-flags.sh
	chkSymbolic=`file $pahole | grep symbolic | wc -l `
	if [ $chkSymbolic -eq 1 ]; then
		link=`readlink -f $pahole`
		sed -i 's/return/exit/' $link
		if [ $? -eq 0 ]; then
			echo successful "$link"
		fi
	else
		sed -i 's/return/exit/' $pahole
		if [ $? -eq 0 ]; then
			echo successful "$pahole"
		fi
	fi
fi	

if [ "$OSDIST" = "RHEL" -a $OSVER -lt 6 ]; then
	echo "["$OSDIST"] version["$OSVER"] does not support."
	exit 1
elif [ "$OSDIST" = "CentOS" -a $OSVER -lt 6 ]; then
	echo "["$OSDIST"] version["$OSVER"] does not support."
	exit 1
elif [ "$OSDIST" = "Asianux" -a $OSVER -lt 7 ]; then
	echo "["$OSDIST"] version["$OSVER"] does not support."
	exit 1
elif [ "$OSDIST" = "Oracle" -a $OSVER -lt 6 ]; then
	echo "["$OSDIST"] version["$OSVER"] does not support."
	exit 1
elif [ "$OSDIST" = "AlmaLinux" -a $OSVER -lt 8 ]; then
	echo "["$OSDIST"] version["$OSVER"] does not support."
	exit 1
elif [ "$OSDIST" = "MIRACLE" -a $OSVER -lt 8 ]; then
	echo "["$OSDIST"] version["$OSVER"] does not support."
	exit 1
elif [ "$OSDIST" = "Rocky" -a $OSVER -lt 8 ]; then
	echo "["$OSDIST"] version["$OSVER"] does not support."
	exit 1
elif [ "$OSDIST" = "openSUSE" -a $OSVER -lt 15 ]; then
	echo "["$OSDIST"] version["$OSVER"] does not support."
	exit 1
elif [ "$OSDIST" = "SLES" -a $OSVER -lt 15 ]; then
	echo "["$OSDIST"] version["$OSVER"] does not support."
	exit 1
elif [ "$OSDIST" = "SLED" -a $OSVER -lt 15 ]; then
	echo "["$OSDIST"] version["$OSVER"] does not support."
	exit 1
elif [ "$OSDIST" = "Debian" -a $OSVER -lt 9 ]; then
	echo "["$OSDIST"] version["$OSVER"] does not support."
	exit 1
elif [ "$OSDIST" = "Ubuntu" -a $OSVER -lt 18 ]; then
	echo "["$OSDIST"] version["$OSVER"] does not support."
	exit 1
elif [ "$OSDIST" = "Amazon" -a $OSVER -lt 2 ]; then
	echo "["$OSDIST"] version["$OSVER"] does not support."
	exit 1
fi

if [ "$OSDIST" = "Ubuntu" ] || [ "$OSDIST" = "Debian" ]; then
	chk_dash=`ls -l /bin/sh  | grep dash | wc -l`
	if [ $chk_dash -eq 1 ]; then
		echo "dash dash/sh boolean false" | debconf-set-selections
		dpkg-reconfigure dash --frontend=noninteractive dash
		echo "Change default shellscript dash--->bash."
	fi
fi

bit=`arch | grep -o x86_64`
if [ $? = 1 ]; then
	echo " 32bit OS does not support."
	exit 1
fi

# Script requirment packages
if [ "$OSDIST" = "openSUSE" ] || [ "$OSDIST" = "SLES" ] || [ "$OSDIST" = "SLED" ]; then
	repo_enable=`zypper repos -E | grep Yes | wc -l`
	inst_com="zypper --non-interactive"
	req_wget=`rpm -qa | grep -w wget | wc -l`
	if [ $repo_enable -eq 0 ]; then
		echo -e "${color_red}All repository are disabled.${color_off}"
		echo -e "${color_red}Create Local repository.${color_off}"
		pre_suse_repo
		sleep 3
		$inst_com install wget
		if [ $? -ne 0 ]; then
			echo -e "${color_red}This script requires wget command.${color_off}"
			echo -e "${color_red}Please install wget rpm.${color_off}"
			exit
		fi
	else
		if [ $req_wget -eq 0 ]; then
			$inst_com install wget
			if [ $? -ne 0 ]; then
				echo -e "${color_red}This script requires wget command.${color_off}"
				echo -e "${color_red}Please install wget rpm.${color_off}"
				exit
			fi
		fi
	fi
elif [ "$OSDIST" = "Debian" ] || [ "$OSDIST" = "Ubuntu" ]; then
	repo_enable=`cat /etc/apt/sources.list | grep -e "main" -e "http" -e "cdrom" | grep -v "#" | wc -l`
	inst_com="apt install -y"
	req_wget=`dpkg-query -l wget | grep ii | wc -l`
	if [ $repo_enable -eq 0 ]; then
		echo -e "${color_red}All repository are disabled.${color_off}"
		echo -e "${color_red}Create Local repository.${color_off}"
		pre_debian_repo
		sleep 3
		$inst_com wget
		if [ $? -ne 0 ]; then
			echo -e "${color_red}This script requires wget command.${color_off}"
			echo -e "${color_red}Please install wget dpkg.${color_off}"
			exit
		fi
	else 
		if [ $req_wget -eq 0 ]; then
			pre_debian_repo
			$inst_com wget
			if [ $? -ne 0 ]; then
				echo -e "${color_red}This script requires wget  command.${color_off}"
				echo -e "${color_red}Please install wget dpkg.${color_off}"
				exit
			fi
		fi
	fi
else
	inst_com="yum -y"
	req_wget=`rpm -qa | grep -w wget | wc -l`
	if [ $req_wget -eq 0 ]; then
		$inst_com install -q wget
		if [ $? -ne 0 ]; then
			echo -e "${color_red}This script requires wget command.${color_off}"
			echo -e "${color_red}Please install wget rpm.${color_off}"
			exit
		fi
	fi
fi

# Debian and ubuntu disable auto kernel update.
# Because, there is a bug in dattobd.

if [ "$OSDIST" = "Ubuntu" ]; then
chk_hold=`apt-mark showhold | grep -e linux-generic -e linux-image-generic -e linux-headers-generic | wc -l`
	if [ $chk_hold -eq 3 ]; then
		echo "Already [apt-mark hold linux-generic linux-image-generic linux-headers-generic]"
	else
		apt-mark hold linux-generic linux-image-generic linux-headers-generic
	fi
elif [ "$OSDIST" = "Debian" ]; then
chk_hold=`apt-mark showhold | grep -e linux-image -e linux-headers | wc -l`
	if [ $chk_hold -eq 2 ]; then
		echo "Already [apt-mark hold linux-image linux-headers]"
	else
		apt-mark hold linux-image-amd64 linux-headers-amd64
	fi
fi

}

ask_YN ()
{
	local ans="invalid"
	local default=Y
	local defstr="[Y/n]"

	if [[ "$2" = [nN] ]]; then
		default="N"
		defstr="[y/N]"
	fi

	echo -n "$1 $defstr "
	read -n1 ans
	while [[ $ans != *[yYnN] ]]; do
		echo
		echo -n "$1 $defstr "
		read -n1 ans
	done
	[ "$ans" = "" ] && ans=$default
	echo

	RC_ask_YN="Y"
	[[ $ans = [nN] ]] && RC_ask_YN="N"
}

Check_kernel(){
kver+=(`uname -r | grep -o "[0-9]*"`)
if [ ${kver[0]} -ge 5 ]; then
	if [ ${kver[1]} -ge 16 ]; then
		echo -e "${color_red}Kernel Version ["$(uname -r)"] does not support.${color_off}"
		echo -e "${color_red}Please install kernel 5.15.x or less.${color_off}"
		echo -e "${color_red}If 5.15.x or less kernel installed, please boot the old kernel.${color_off}"
		echo -e "${color_red}Then, please uninstall kernel 5.16 or later.${color_off}"
		exit
	fi
fi
}

set_sudo(){
usr=`who | awk '{print $1}'`
if [ -n "$usr" ]; then
	chk_sudo=`cat /etc/group | grep sudo | grep $usr | wc -l`
	if [ $chk_sudo -ne 1 ]; then
		gpasswd -a $usr sudo
	fi
fi
}

Check_fs(){
chk_btrfs=`lsblk -n -l -o FSTYPE | grep btrfs | wc -l`
if [ $chk_btrfs -ne 0 ]; then
	echo -e "${color_red}[btrfs] file system has been detected.${color_off}"
	echo -e "${color_red}AIP2022 does not support btrfs.${color_off}"
	echo -e "${color_red}AIP2022 supports ext3/4, xfs.${color_off}"
	exit
fi
}

#See RHBA-2020:4473-BugFixAdvisory for this issue.
#systemd-udev-239-33.el8 breaks dracut: /usr/lib/dracut-init.sh: line 460:[:too many arguments
Chk_dracut_ver()	
{
udev_ver=`udevadm --version | wc -w`
if [ $udev_ver -eq 2 ]; then
	exist_fix=`cat /lib/dracut/dracut-init.sh | grep "udevadm --version" | grep "read v" | wc -l`
	if [ $exist_fix -ne 1 ]; then
		cp /lib/dracut/dracut-init.sh /tmp/
		sed -i 's/udevadm\ \--version/udevadm\ \--version\ \|\ \{\ read v _ \;\ echo\ \$v\ \;\ \}/' /lib/dracut/dracut-init.sh
		if [ $? -ne 0 ]; then
			\cp -f /tmp/dracut-init.sh /lib/dracut/dracut-init.sh
			echo "Failed to fixe dracut-init.sh"
			exit 1
		fi
	fi
fi
}

Check_Dist
Check_fs

print_head()
{
	clear
	echo -e "${color_blue_back}******************************************************************${color_off}"
	echo -e "${color_blue_back}**      ActiveImage Protector 2022 Linux installation tool      **${color_off}"
	echo -e "${color_blue_back}******************************************************************${color_off}"
	echo ""
}

print_head
echo "** `date '+%Y-%m-%d %H:%M:%S'` -Ver $ScVer START"
if [ "$OSDIST" = "Debian" ] && [ "$OSVER" = "94" ]; then
	echo "$OSDIST 9"
else
	echo "$OSDIST $OSVER"
fi
echo System checking....

Check_kernel

oracle_repo() {
#if [ "$OSDIST" = "CentOS" ]; then
	echo [ol_latest] >> /etc/yum.repos.d/ol_temp.repo
	echo name=ol_latest >> /etc/yum.repos.d/ol_temp.repo
	if [ ${OSVER} -eq 6 ]; then
		echo baseurl=https://yum.oracle.com/repo/OracleLinux/OL6/latest/x86_64/ >> /etc/yum.repos.d/ol_temp.repo
	elif [ ${OSVER} -eq 7 ]; then
		echo baseurl=https://yum.oracle.com/repo/OracleLinux/OL7/latest/x86_64/ >> /etc/yum.repos.d/ol_temp.repo
	elif [ ${OSVER} -eq 8 ]; then
		echo baseurl=https://yum.oracle.com/repo/OracleLinux/OL8/baseos/latest/x86_64/ >> /etc/yum.repos.d/ol_temp.repo
		echo [ol_app] >> /etc/yum.repos.d/ol_app_temp.repo
		echo name=ol_app >> /etc/yum.repos.d/ol_app_temp.repo
		echo baseurl=https://yum.oracle.com/repo/OracleLinux/OL8/appstream/x86_64/ >> /etc/yum.repos.d/ol_app_temp.repo
		echo enable=1 >> /etc/yum.repos.d/ol_app_temp.repo
		echo gpgcheck=0 >> /etc/yum.repos.d/ol_app_temp.repo
	elif [ ${OSVER} -eq 9 ]; then
		echo baseurl=https://yum.oracle.com/repo/OracleLinux/OL9/baseos/latest/x86_64/ >> /etc/yum.repos.d/ol_temp.repo
		echo [ol_app] >> /etc/yum.repos.d/ol_app_temp.repo
		echo name=ol_app >> /etc/yum.repos.d/ol_app_temp.repo
		echo baseurl=https://yum.oracle.com/repo/OracleLinux/OL9/appstream/x86_64/ >> /etc/yum.repos.d/ol_app_temp.repo
		echo enable=1 >> /etc/yum.repos.d/ol_app_temp.repo
		echo gpgcheck=0 >> /etc/yum.repos.d/ol_app_temp.repo
	fi
	echo enable=1 >> /etc/yum.repos.d/ol_temp.repo
	echo gpgcheck=0 >> /etc/yum.repos.d/ol_temp.repo

	chk_ol=`rpm -qa | grep oraclelinux-release | wc -l`
	if [ $chk_ol -eq 0 ]; then
		if [ ${OSVER} -eq 6 ]; then
			echo "Checking yum repository ....."
			wget -q https://yum.oracle.com/RPM-GPG-KEY-oracle-ol6 -O /etc/pki/rpm-gpg/RPM-GPG-KEY-oracle
			gpg --quiet --with-fingerprint /etc/pki/rpm-gpg/RPM-GPG-KEY-oracle
			$inst_com install -q oraclelinux-release-el6
		elif [ ${OSVER} -eq 7 ]; then
			echo "Checking yum repository ....."
			wget -q https://yum.oracle.com/RPM-GPG-KEY-oracle-ol7 -O /etc/pki/rpm-gpg/RPM-GPG-KEY-oracle
			gpg --quiet --with-fingerprint /etc/pki/rpm-gpg/RPM-GPG-KEY-oracle
			$inst_com install -q oraclelinux-release-el7
		elif [ ${OSVER} -eq 8 ]; then
			echo "Checking yum repository ....."
 			wget -q https://yum.oracle.com/RPM-GPG-KEY-oracle-ol8 -O /etc/pki/rpm-gpg/RPM-GPG-KEY-oracle
              	gpg -q --import /etc/pki/rpm-gpg/RPM-GPG-KEY-oracle
			$inst_com install -q oraclelinux-release-el8
		fi
	fi
#fi
if [ "$OSVER" = "8" ]; then
	sed -i -e 's|rhel8|ol8|g' /etc/dnf/modules.d/*.module
fi
}

vault_repo_base() {
#no update
if [ "$OSDIST" = "CentOS" ]; then
	echo [vault_base] >> /etc/yum.repos.d/vault.repo
	echo name=vault_base >> /etc/yum.repos.d/vault.repo
	if [ ${OSVER} -eq 7 ]; then
		echo baseurl=http://vault.centos.org/$release_ver1/os/$plat/ >> /etc/yum.repos.d/vault.repo
	elif [ ${OSVER} -eq 8 ]; then 
		echo baseurl=http://vault.centos.org/$release_ver1/BaseOS/$plat/os/ >> /etc/yum.repos.d/vault.repo
	else 
		echo baseurl=http://vault.centos.org/$os_ver/os/$plat/ >> /etc/yum.repos.d/vault.repo
	fi
	echo enable=1 >> /etc/yum.repos.d/vault.repo
	echo gpgcheck=0 >> /etc/yum.repos.d/vault.repo
fi
}

vault_repo_base2() {
#major update
if [ "$OSDIST" = "CentOS" ]; then
	echo [vault_base] >> /etc/yum.repos.d/vault.repo
	echo name=vault_base >> /etc/yum.repos.d/vault.repo
	if [ ${OSVER} -eq 7 ]; then
		echo baseurl=http://vault.centos.org/$release_ver1/os/$plat/ >> /etc/yum.repos.d/vault.repo
	elif [ ${OSVER} -eq 8 ]; then 
		echo baseurl=http://vault.centos.org/$release_ver1/BaseOS/$plat/os/ >> /etc/yum.repos.d/vault.repo
	else 
		echo baseurl=http://vault.centos.org/$os_ver/os/$plat/ >> /etc/yum.repos.d/vault.repo
	fi
	echo enable=1 >> /etc/yum.repos.d/vault.repo
	echo gpgcheck=0 >> /etc/yum.repos.d/vault.repo
fi
}

vault_repo_update() {
if [ "$OSDIST" = "CentOS" ]; then
	echo [vault_updates] >> /etc/yum.repos.d/vault.repo
	echo name=vault_updates >> /etc/yum.repos.d/vault.repo
	if [ ${OSVER} -eq 7 ]; then
		echo baseurl=http://vault.centos.org/$release_ver1/updates/$plat/ >> /etc/yum.repos.d/vault.repo
	elif [ ${OSVER} -eq 8 ]; then 
		echo baseurl=http://vault.centos.org/$release_ver1/BaseOS/$plat/os/ >> /etc/yum.repos.d/vault.repo
	else
		echo baseurl=http://vault.centos.org/$release_ver/updates/$plat/ >> /etc/yum.repos.d/vault.repo
	fi
	echo enable=1 >> /etc/yum.repos.d/vault.repo
	echo gpgcheck=0 >> /etc/yum.repos.d/vault.repo
fi
}

cent6_repo_res(){
if [ "$OSDIST" = "CentOS" ]; then
	if [ -f /etc/yum.repos.d/CentOS-Base.repo.bk ]; then
		mv /etc/yum.repos.d/CentOS-Base.repo.bk /etc/yum.repos.d/CentOS-Base.repo
	fi
	if [ ${OSVER} -eq 6 ]; then
		if [ -f /etc/yum.repos.d/epel.repo.bk ]; then
			mv /etc/yum.repos.d/epel.repo.bk /etc/yum.repos.d/epel.repo
		fi
	fi
fi
}

cent6_repo(){
if [ "$OSDIST" = "CentOS" ]; then
	min_ver=`cat /etc/system-release | grep -oE "([0-9])\.([0-9]*)"`
	if [ ${OSVER} -eq 6 ]; then
		if [ -f /etc/yum.repos.d/CentOS-Base.repo ]; then
			mv /etc/yum.repos.d/CentOS-Base.repo /etc/yum.repos.d/CentOS-Base.repo.bk
		fi
		if [ -f /etc/yum.repos.d/epel.repo ]; then
			mv /etc/yum.repos.d/epel.repo /etc/yum.repos.d/epel.repo.bk
		fi
		if [ -f /etc/yum.repos.d/CentOS-Vault.repo ]; then
			sed -i "s/6.0/$min_ver/" /etc/yum.repos.d/CentOS-Vault.repo
			sed -ri "/^\[C$min_ver-base\]$/,/^\[/ s/^(enabled)( *)=.*$/\1\2=\21/;
				/^\[C$min_ver-updates\]$/,/^\[/ s/^(enabled)( *)=.*$/\1\2=\21/;
				/^\[C$min_ver-extras\]$/,/^\[/ s/^(enabled)( *)=.*$/\1\2=\21/;
				/^\[C$min_ver-contrib\]$/,/^\[/ s/^(enabled)( *)=.*$/\1\2=\21/;
				/^\[C$min_ver-centosplus\]$/,/^\[/ s/^(enabled)( *)=.*$/\1\2=\21/" /etc/yum.repos.d/CentOS-Vault.repo
		else
HOGE=$(cat << EOF >> /etc/yum.repos.d/CentOS-Vault.repo
[C$min_ver-base]
name=CentOS-$min_ver - Base
baseurl=http://vault.centos.org/$min_ver/os/$(uname -p)/
gpgcheck=0
enabled=1
[C$min_ver-updates]
name=CentOS-$min_ver - Updates
baseurl=http://vault.centos.org/$min_ver/updates/$(uname -p)/
gpgcheck=0
enabled=1
[C$min_ver-extras]
name=CentOS-$min_ver - Extras
baseurl=http://vault.centos.org/$min_ver/extras/$(uname -p)/
gpgcheck=0
enabled=1
EOF
)

		fi
	fi
fi
}

cent6_repo_disable(){
if [ "$OSDIST" = "CentOS" ]; then
	min_ver=`cat /etc/system-release | grep -oE "([0-9])\.([0-9]*)"`
	if [ ${OSVER} -eq 6 ]; then
		if [ -f /etc/yum.repos.d/CentOS-Base.repo ]; then
			mv /etc/yum.repos.d/CentOS-Base.repo.bk /etc/yum.repos.d/CentOS-Base.repo
		fi
		if [ -f /etc/yum.repos.d/epel.repo ]; then
			mv /etc/yum.repos.d/epel.repo.bk /etc/yum.repos.d/epel.repo
		fi
		if [ -f /etc/yum.repos.d/CentOS-Vault.repo ]; then
			sed -i "s/6.0/$min_ver/" /etc/yum.repos.d/CentOS-Vault.repo
			sed -ri "/^\[C$min_ver-base\]$/,/^\[/ s/^(enabled)( *)=.*$/\1\2=\20/;
				/^\[C$min_ver-updates\]$/,/^\[/ s/^(enabled)( *)=.*$/\1\2=\20/;
				/^\[C$min_ver-extras\]$/,/^\[/ s/^(enabled)( *)=.*$/\1\2=\20/;
				/^\[C$min_ver-contrib\]$/,/^\[/ s/^(enabled)( *)=.*$/\1\2=\20/;
				/^\[C$min_ver-centosplus\]$/,/^\[/ s/^(enabled)( *)=.*$/\1\2=\20/" /etc/yum.repos.d/CentOS-Vault.repo
		else
HOGE=$(cat << EOF >> /etc/yum.repos.d/CentOS-Vault.repo
[C$min_ver-base]
name=CentOS-$min_ver - Base
baseurl=http://vault.centos.org/$min_ver/os/$(uname -p)/
gpgcheck=0
enabled=0
[C$min_ver-updates]
name=CentOS-$min_ver - Updates
baseurl=http://vault.centos.org/$min_ver/updates/$(uname -p)/
gpgcheck=0
enabled=0
[C$min_ver-extras]
name=CentOS-$min_ver - Extras
baseurl=http://vault.centos.org/$min_ver/extras/$(uname -p)/
gpgcheck=0
enabled=0
EOF
)

		fi
	fi
fi

}

vault_remove() {
	if [ -f /etc/yum.repos.d/vault.repo ]; then
		rm -rf /etc/yum.repos.d/vault.repo
	fi
}

DVD_remove() {
	if [ -f /etc/yum.repos.d/DVD.repo ]; then
		rm -rf /etc/yum.repos.d/DVD*.repo
	fi
}

ol_remove() {
	if [ -f /etc/yum.repos.d/ol_temp.repo ]; then
		rm -rf /etc/yum.repos.d/ol_temp.repo
		#rm -rf /etc/yum.repos.d/oracle*.repo
		#rm -rf /etc/yum.repos.d/uek*.repo
		#rm -rf /etc/yum.repos.d/virt-ol*.repo
	fi
	if [ -f /etc/yum.repos.d/ol_app_temp.repo ]; then
		rm -rf /etc/yum.repos.d/ol_app_temp.repo
		#rm -rf /etc/yum.repos.d/oracle*.repo
		#rm -rf /etc/yum.repos.d/uek*.repo
		#rm -rf /etc/yum.repos.d/virt-ol*.repo
	fi
	
	# include oracle*.repo uek*.repo virt-ol*.repo, but exclude OracleLinux. 
	if [ "$OSDIST" != "Oracle" ]; then
		rpm --version >/dev/null 2>&1
		if [ $? -eq 0 ]; then
			if [ "$OSDIST" != "openSUSE" ] || [ "$OSDIST" != "SLES" ] || [ "$OSDIST" != "SLED" ]; then
				e_ora_rpm=`rpm -qa | grep oraclelinux-release | wc -l`
				if [ $e_ora_rpm -ne 0 ]; then
					ora_rpm+=`rpm -qa | grep oraclelinux-release`
					for str in ${ora_rpm[@]}; do
						rpm -e $str
					done
				fi
			fi
		fi
	fi
if [ "$OSVER" = "8" ]; then
	sed -i -e 's|ol8|rhel8|g' /etc/dnf/modules.d/*.module
fi
}
google_remove() {
	if [ -f /etc/yum.repos.d/google.chrome.repo ]; then
		rm -rf /etc/yum.repos.d/google.chrome.repo
	fi
}
yum_proxy() {

	export http_proxy="http://$IPadd_name:$port"
	export proxy_username="$user"
	export proxy_password="$pass"
}

repo() {
	echo [DVD] >> /etc/yum.repos.d/DVD.repo
	echo name=DVD >> /etc/yum.repos.d/DVD.repo
	echo baseurl=file://$dvd_path/ >> /etc/yum.repos.d/DVD.repo
	echo enable=1 >> /etc/yum.repos.d/DVD.repo
}

repo_ent5() {
	echo [DVD] >> /etc/yum.repos.d/DVD.repo
	echo name=DVD >> /etc/yum.repos.d/DVD.repo
	echo baseurl=file://$dvd_path/Server >> /etc/yum.repos.d/DVD.repo
	echo enable=1 >> /etc/yum.repos.d/DVD.repo
}

try_another(){
if [ "$OSDIST" = "CentOS" ]; then
		$inst_com --disablerepo=* --enablerepo=vault_base --nogpgcheck install $MISSING1
		if [ $? -eq 1 ]; then
			echo -e "${color_red}yum install is failed in vault repository.${color_off}"
			echo -e "${color_red}Try another repository.${color_off}"
		fi
		ins_check_devel2=`rpm -qa kernel-devel | wc -l`
		if [ $ins_check_devel2 = 0 ]; then
			vault_remove
			oracle_repo
			$inst_com install $MISSING1
			if [ $? -eq 1 ]; then
				echo -e "${color_red}yum install is failed.${color_off}"
				echo -e "${color_red}Please get "$MISSING1" in another way.${color_off}"
				echo -e "${color_red}Please install "$MISSING1".${color_off}"
				echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
				exit 1
			fi
		fi
else
	$inst_com install $MISSING1
	if [ $? -eq 1 ]; then
		echo -e "${color_red}yum install is failed.${color_off}"
		echo -e "${color_red}Try another repository.${color_off}"
		oracle_repo
		$inst_com install $MISSING1
		if [ $? -eq 1 ]; then
			echo -e "${color_red}yum install is failed.${color_off}"
			echo -e "${color_red}Please get "$MISSING1" in another way.${color_off}"
			echo -e "${color_red}Please install "$MISSING1".${color_off}"
			echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
			exit 1
		fi
	fi
fi
}

ker_dev_not_inst_match() {
	
	echo -e "${color_blue}kernel-devel is not installed.${color_off}"
	echo -e "${color_blue}OS Ver and kernel ver is match.${color_off}"
	uname -r | grep -oE '\-[0-9]+.[0-9]+.[0-9]+'
	if [ $? -eq 0 ]; then
		echo "Minor update"
		if [ "$OSDIST" != "RHEL" ]; then
			vault_repo_update
		fi
		echo "$MISSING1 install"
		try_another
	else
		echo "Base"
		if [ "$OSDIST" != "RHEL" ]; then
			vault_repo_base
		fi
		echo "$MISSING1 install"
		try_another
	fi
}

ker_dev_not_inst_not_match() {

	echo -e "${color_blue}kernel-devel is not installed.${color_off}"
	echo -e "${color_blue}OS Ver and kernel ver is not match.${color_off}"
	echo "Major update"		
	echo "Base"
	if [ "$OSDIST" != "RHEL" ]; then
		vault_repo_base2
	fi
	echo "$MISSING1 install"
	try_another
}

ker_dev_inst_match() {

	echo -e "${color_blue}kernel-devel is installed.${color_off}"
	echo -e "${color_blue}OS Ver and kernel ver is match.${color_off}"
	ker_ver=`uname -r`
	ker_dev_ver=`rpm -qa kernel-devel | grep -oE '[0-9]+.[0-9]+.[0-9]+.-[0-9]+.*'`
	echo "Checking kernel and kernel-devel...."
	if [ $ker_ver = $ker_dev_ver ]; then
		echo "kernel and kernel-devel is match."
	else 
		echo "kernel and kernel-devel is not match."
		echo "Checking kernel update...."
		uname -r | grep -oE '\-[0-9]+.[0-9]+.[0-9]+'
		if [ $? -eq 0 ]; then
			echo "Minor update"
			if [ "$OSDIST" != "RHEL" ]; then
				vault_repo_update
			fi
			echo "$MISSING1 install"
			try_another
		else
			echo "Base"
			if [ "$OSDIST" != "RHEL" ]; then
				vault_repo_base
			fi
			echo "$MISSING1 install"
			try_another
		fi
	fi
}

ker_dev_inst_not_match() {

	echo -e "${color_blue}kernel-devel is installed.${color_off}"
	echo -e "${color_blue}OS Ver and kernel ver is not match.${color_off}"
	ker_ver=`uname -r`
	ker_dev_ver=`rpm -qa kernel-devel | grep -oE '[0-9]*\.[0-9]*\.[0-9]*\-[0-9]*'`
	echo "Checking kernel and kernel-devel...."
	if [ $ker_ver = $ker_dev_ver ]; then
		echo "kernel and kernel-devel is match."
	else 
		echo "kernel and kernel-devel is not match."
		echo "Checking kernel update...."
		uname -r | grep -oE '\-[0-9]+.[0-9]+.[0-9]+'
		if [ $? -eq 0 ]; then
			echo "Major update"
			if [ "$OSDIST" != "RHEL" ]; then
				vault_repo_update
			fi	
			echo "$MISSING1 install"
			try_another
		else
			echo "Base"
			if [ "$OSDIST" != "RHEL" ]; then
				vault_repo_base2
			fi
			echo "$MISSING1 install"
			try_another
		fi
	fi
}


Check_sb()	#Check SecureBoot
{
if [ ${OSVER} -ge 7 ]; then
	sb=`mokutil --sb-state 2>/dev/null | grep "SecureBoot enabled" | wc -l`
	if [ $sb -eq 1 ]; then
		echo "This system is enabled SecureBoot."
		echo "AIP Linux dose not support Secureboot."
		echo "Please disable SecureBoot and try install."
		exit 1
	fi
else
	sb=`mokutil --sb-state 2>/dev/null | grep "SecureBoot enabled" | wc -l`
	if [ $sb -eq 1 ]; then
		echo "This system is enabled SecureBoot."
		echo "AIP Linux dose not support Secureboot."
		echo "Please disable SecureBoot and try install."
		exit 1
	fi
fi
}

Check_GLIB()
{
	GLIBCXX=`strings /usr/lib64/libstdc++.so.6 | grep "GLIBCXX_3.4.15" | wc -l`
	GLIBC=`strings /lib64/libc.so.6 | grep "GLIBC_2.7" | wc -l`
	if [ $GLIBCXX -eq 0 -o $GLIBC -eq 0 ]; then
		echo -e "${color_red}lease update [gcc glibc] packages.${color_off}"
	fi
}


#RHEL Only Function
Check_subsc()
{
	if [ "$OSDIST" = "RHEL" ]; then
		if [ ${OSVER} -eq 5 ]; then
			subsc_stat=`subscription-manager list | grep Status: | awk '{print $2,$3}'`
			if [ "$subsc_stat" = "Not Subscribed" ]; then
				subsc=1	#subscription unknown
			else
				subsc=0	#subscription ok
			fi
		else
			subsc_stat=`subscription-manager list --matches="Red Hat*" | grep Status: | awk '{print $2}'`
			if [ $subsc_stat = "Unknown" ]; then
				subsc=1	#subscription unknown
			else
				subsc=0	#subscription ok
			fi
		fi
		echo "Subscription Status: "$subsc_stat
	fi
}

repo() 
{
	echo [DVD] >> /etc/yum.repos.d/DVD.repo
	echo name=DVD >> /etc/yum.repos.d/DVD.repo
	echo baseurl=file://$dvd_path/ >> /etc/yum.repos.d/DVD.repo
	echo enabled=1 >> /etc/yum.repos.d/DVD.repo
}

repo8() 
{
	echo [DVD] >> /etc/yum.repos.d/DVD.repo
	echo name=DVD >> /etc/yum.repos.d/DVD.repo
	echo baseurl=file://$dvd_path/BaseOS/ >> /etc/yum.repos.d/DVD.repo
	echo enabled=1 >> /etc/yum.repos.d/DVD.repo
	echo gpgcheck=0 >> /etc/yum.repos.d/DVD.repo
	echo ""
	echo [DVD1] >> /etc/yum.repos.d/DVD.repo
	echo name=DVD1 >> /etc/yum.repos.d/DVD.repo
	echo baseurl=file://$dvd_path/AppStream/ >> /etc/yum.repos.d/DVD.repo
	echo enabled=1 >> /etc/yum.repos.d/DVD.repo
	echo gpgcheck=0 >> /etc/yum.repos.d/DVD.repo
}


Check_DVD_path()
{
	if [ ! -e "$dvd_path" ]; then
		echo "[$dvd_path]" "Path not found."
		exit
	fi
}

Check_internet()
{
	wget --spider -q -nv --timeout 10 -t 1 https://www.google.com
	if [ $? -eq 0 ]; then
		net_ok=0	#connect ok
	else
		net_ok=1	#not connect
	fi
}

try_another_rpm(){
	echo -e "${color_red}yum install is failed.${color_off}"
	echo -e "${color_red}Try another repository.${color_off}"
	oracle_repo
	$inst_com install $MISSING
	if [ $? -eq 1 ]; then
		echo -e "${color_red}yum install is failed.${color_off}"
		echo -e "${color_red}Please get "$MISSING" in another way.${color_off}"
		echo -e "${color_red}Please install "$MISSING".${color_off}"
		echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
	fi
}

Cent6_later_net() 
{
	echo -e "${color_blue}Install the required rpm packages using yum command.${color_off}" 	
	if [ $Agent -ne 1 ]; then
		if [ $OSVER -eq 7 ]; then
			yum list installed | grep redhat-lsb >> /dev/null
			if [ $? -eq 1 ]; then	
				$inst_com install redhat-lsb.x86_64	
			else
				$inst_com upgrade redhat-lsb.x86_64	
			fi
		fi
		if [ $OSVER -eq 8 ]; then
			yum list installed | grep redhat-lsb >> /dev/null
			if [ "$?" = 1 ]; then	
				$inst_com install redhat-lsb.x86_64	
			else
				$inst_com upgrade redhat-lsb.x86_64	
			fi
		fi
	fi
	if [ $OSVER -eq 6 ]; then
		$inst_com install $MISSING
		if [ $? -eq 1 ]; then
			cent6_repo_disable
			try_another_rpm
		fi
	fi
	if [ $OSVER -ge 7 ]; then
		$inst_com update $EXIST
		$inst_com install $MISSING
		if [ $? -eq 1 ]; then
			try_another_rpm
		fi
	fi
}

Amazon_net() 
{
	echo -e "${color_blue}Install the required rpm packages using yum command.${color_off}" 	
	if [ $OSVER -eq 2 ]; then
		$inst_com update $EXIST
		$inst_com install $MISSING
	fi
}

Cent6_7_nonet() 
{
	echo "=== Internet connection failed. Please insert the OS DVD."
	echo -n "Input DVD mount path [e.g.: /run/media/root/CentOS 7 x86_64] :"
	read dvd_path
	Check_DVD_path
	dvd_path=`echo "$dvd_path" | sed -e 's/ /%20/g'`
	$(repo)
	echo -e "${color_blue}Install the required rpm packages using yum command.${color_off}"
	if [ $Agent -ne 1 ]; then
		yum list installed | grep redhat-lsb >> /dev/null
		if [ "$?" = 1 ]; then	
			$inst_com --disablerepo=* --enablerepo=DVD --nogpgcheck install redhat-lsb.x86_64
		else
			$inst_com --disablerepo=* --enablerepo=DVD --nogpgcheck upgrade redhat-lsb.x86_64
		fi
	fi
	$inst_com --disablerepo=* --enablerepo=DVD --nogpgcheck update $EXIST
	$inst_com --disablerepo=* --enablerepo=DVD --nogpgcheck install $MISSING

}

Cent8_nonet() 
{
	echo "=== Internet connection failed. Please insert the OS DVD."
	echo -n "Input DVD mount path [e.g.: /run/media/root/CentOS 7 x86_64] :"
	read dvd_path
	Check_DVD_path
	dvd_path=`echo "$dvd_path" | sed -e 's/ /%20/g'`
	$(repo8)
	echo -e "${color_blue}Install the required rpm packages using dnf command.${color_off}"
	if [ $Agent -ne 1 ]; then
		dnf list --installed | grep redhat-lsb >> /dev/null
		if [ "$?" = 1 ]; then	
			$inst_com --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck install redhat-lsb.x86_64
		else
			$inst_com --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck upgrade redhat-lsb.x86_64
		fi
	fi
	$inst_com --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck update $EXIST
	$inst_com --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck install $MISSING

}

RHEL6_later_net() 
{
	echo "=== Internet connection OK."
	if [ "$subsc" == "0" ]; then
		echo "=== Subscription Status: $subsc_stat."
	fi
	echo -e "${color_blue}Install the required rpm packages using yum command.${color_off}" 			
	if [ $Agent -ne 1 ]; then
		if [ $OSVER -le 7 ]; then
			yum list installed | grep redhat-lsb >> /dev/null
			if [ "$?" = 1 ]; then	
				$inst_com install redhat-lsb.x86_64	
			else
				$inst_com upgrade redhat-lsb.x86_64			
			fi
		fi
		if [ $OSVER -eq 8 ]; then
			yum list installed | grep redhat-lsb >> /dev/null
			if [ "$?" = 1 ]; then	
				$inst_com install redhat-lsb.x86_64			
			else
				$inst_com upgrade redhat-lsb.x86_64			
			fi
		fi
	fi
	if [ $OSVER -le 7 ]; then
		$inst_com update $EXIST
		$inst_com install $MISSING
		if [ $? -eq 1 ]; then
			try_another_rpm
		fi
	fi
	if [ $OSVER -ge 8 ]; then
		$inst_com update $EXIST
		$inst_com install $MISSING
		if [ $? -eq 1 ]; then
			try_another_rpm
		fi
	fi

}

RHEL6_7_nonet() 
{	
	if [ "$net_ok" == "0" ]; then
		echo "=== Internet connection OK."
	else
		echo "=== Internet connection failed. Please insert the OS DVD."
	fi
	if [ "$subsc" == "1" ]; then
		echo "=== Subscription Status: $subsc_stat. Please insert the OS DVD."
	else
		echo "=== Subscription Status: $subsc_stat."
	fi
	echo -n "Input DVD mount path [e.g.: run/media/root/CentOS 7 x86_64] :"
	read dvd_path
	Check_DVD_path
	dvd_path=`echo "$dvd_path" | sed -e 's/ /%20/g'`
	$(repo)
	echo -e "${color_blue}Install the required module using yum command.${color_off}"
	if [ $Agent -ne 1 ]; then
		yum list installed | grep redhat-lsb >> /dev/null
		if [ "$?" = 1 ]; then	
			$inst_com --disablerepo=* --enablerepo=DVD --nogpgcheck install redhat-lsb.x86_64
		else
			$inst_com --disablerepo=* --enablerepo=DVD --nogpgcheck upgrade redhat-lsb.x86_64
		fi
	fi
	$inst_com --disablerepo=* --enablerepo=DVD --nogpgcheck update $EXIST
	$inst_com --disablerepo=* --enablerepo=DVD --nogpgcheck install $MISSING

}

RHEL8_nonet() 
{
	if [ "$net_ok" == "0" ]; then
		echo "=== Internet connection OK."
	else
		echo "=== Internet connection failed. Please insert the OS DVD."
	fi
	if [ "$subsc" == "1" ]; then
		echo "=== Subscription Status: $subsc_stat. Please insert the OS DVD."
	else
		echo "=== Subscription Status: $subsc_stat."
	fi
	echo -n "Input DVD mount path [e.g.: /run/media/root/CentOS 7 x86_64] :"
	read dvd_path
	if [ ! -e $dvd_path ]; then
		echo "Path not found."
	fi
	dvd_path=`echo "$dvd_path" | sed -e 's/ /%20/g'`
	$(repo8)
	echo -e "${color_blue}Install the required module using yum command.${color_off}"
	if [ $Agent -ne 1 ]; then
		dnf list --installed | grep redhat-lsb >> /dev/null
		if [ "$?" = 1 ]; then	
			$inst_com --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck install redhat-lsb.x86_64
		else
			$inst_com --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck upgrade redhat-lsb.x86_64
		fi
	fi
	$inst_com --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck update $EXIST
	$inst_com --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck install $MISSING

}

SUSE_net() 
{
	echo -e "${color_blue}Install the required rpm packages using zypper command.${color_off}" 	
		#$inst_com update $EXIST
		$inst_com install $MISSING
}

SUSE_nonet() 
{
repo_exist=`zypper repos -E | grep Yes | wc -l`
	if [ $repo_exist -eq 0 ]; then
		echo "=== Internet connection failed. Please insert the OS DVD."
		echo -n "Input DVD mount path [e.g.: /run/media/root/SLE-15-SP2-Full-x86_64209.21.001] :"
		read dvd_path
		if [ ! -e $dvd_path ]; then
			echo "Path not found."
		fi
	fi
	suse_repo
	echo -e "${color_blue}Install the required rpm packages using zypper command.${color_off}"
	$inst_com install $MISSING

}

Debian_net() 
{
	echo -e "${color_blue}Install the required rpm packages using apt command.${color_off}"
	$inst_com $MISSING
}

Debian_nonet() 
{
	debian_repo
	echo -e "${color_blue}Install the required dpkg packages using apt command.${color_off}"
	$inst_com $MISSING

}

suse_repo()
{
repo_arry2=()
repo_arry3=()

repodir="/etc/zypp/repos.d"

repos_count=`ls $dvd_path | grep -c Module`
if [ $repos_count -ne 0 ]; then
repos=`ls $dvd_path | grep Module`
	repo_arry2+=($repos)
	BK_IFS=$IFS
	IFS=$'\n'
	repo_count=`echo "${repo_arry2[*]}" | grep -e Basesystem -e Desktop-Applications -e Development-Tools | wc -l`
	if [ $repo_count -eq 3 ]; then
		repo_arry3+=(`echo "${repo_arry2[*]}" | grep -e Basesystem -e Desktop-Applications -e Development-Tools`)
		for i in ${repo_arry3[*]}
		do
			zypper addrepo -G -n DVD $dvd_path DVD-$i
			if [ -f $repodir/DVD-$i.repo ]; then
				sed -i "$ a path=/$i" $repodir/DVD-$i.repo
			fi
		done
	fi
else
	repos=`ls $dvd_path | grep repodata`
	repo_arry+=($repos)
	BK_IFS=$IFS
	IFS=$'\n'
	repo_count=`echo "${repo_arry[*]}" | grep -e repodata | wc -l`
	if [ $repo_count -eq 1 ]; then
		zypper addrepo -G -n DVD $dvd_path DVD
		if [ -f $repodir/DVD.repo ]; then
			sed -i "$ a path=/" $repodir/DVD.repo
		fi
	fi
fi
zypper ref 
IFS=$BK_IFS
}

debian_repo(){

	apt_src="/etc/apt/sources.list"
	sed -i 's/^deb http/\#deb http/' $apt_src
	exit_cdrom_repo=`cat $apt_src | grep "deb cdrom" | wc -l`
	enable_cdrom=`cat $apt_src | grep "deb cdrom" | grep -v "#" | wc -l`
	if [ $exit_cdrom_repo -eq 0 ]; then
		echo "=== Internet connection failed. Please insert the OS DVD."
		echo -n "Input DVD mount path [e.g.: /media/cdrom0] :"
		read dvd_path
		if [ ! -e $dvd_path ]; then
			echo "Path not found."
		fi
		apt-cdrom add
		apt-cdrom ident
		echo "Run apt update....."
		apt update -y >/dev/null 2>&1
	elif [ $exit_cdrom_repo -ne 0 ] || [ $enable_cdrom -ne 0 ]; then
		echo "Enable cdrom repository."
		sed -i 's/^# deb cdrom/\deb cdrom/' $apt_src
		echo "Run apt update....."
		apt update -y >/dev/null 2>&1
	fi
}

number_select()
{
   local title="$1"
   echo ""
   
   local i=0
   local j=1
   while [ "$#" -gt "1" ] ; do
      (( i=i+1 ))
      printf "[$i] $2\n"
      shift
   done

   echo ""
   while [ 1 ] ; do
	  printf "${title} "
	  if [ $i -eq 1 ] ; then
	     printf "[1]:"
	  else
	     printf "[1-${i}]:"
	  fi
	  read op
	  for ((j=1; j<=$i; j=j+1)) ; do
	     [[ "$op" == "$j" ]] && return $op
      done
   done
}

select_options()
{
	while [ 1 ] ; do
		if [ $OSVER -ge 8 ]; then
			number_select "Please select install menu from" "Agent" "GUI" "[1]-[2]All" "Install Kernel-devel" "Install datto driver" "Install AIPLinux" "Uninstall datto driver" "Install AAS" "Uninstall AAS" "help" "exit"
		  	case "$?" in
				1) Agent ;;
				2) GUI ;;
				3) All ;;
				4) pre_kernel_devel
					kernel_devel_install ;;
				5) pre_kernel_devel
					datto_install ;;
				6) pre_kernel_devel
					AIPLinux_install ;;
				7) pre_kernel_devel
					datto_remove ;;
				8) aas ;;
				9) aas_remove ;;
				10) show_help ;;
				11) break ;;
			esac
		else
			number_select "Please select install menu from" "Agent" "GUI" "[1]-[2]All" "Install Kernel-devel" "Install datto driver" "Install AIPLinux" "Uninstall datto driver" "help" "exit"
		  	case "$?" in
				1) Agent ;;
				2) GUI ;;
				3) All ;;
				4) pre_kernel_devel
					kernel_devel_install ;;
				5) pre_kernel_devel
					datto_install ;;
				6) pre_kernel_devel
					AIPLinux_install ;;
				7) pre_kernel_devel
					datto_remove ;;
				8) show_help ;;
				9) break ;;
			esac
		fi
	done
}

oracle_select_options()
{
	while [ 1 ] ; do
		if [ $OSVER -ge 8 ]; then
			number_select "Please select install menu from" "Agent" "GUI" "[1]-[2]All" "Install Kernel-devel" "Install datto driver" "Install AIPLinux" "Uninstall datto driver" "Install AAS" "Uninstall AAS" "help" "exit"
			case "$?" in
				1) Agent ;;
				2) GUI ;;
				3) All ;;
				4) oracle_pre_kernel_devel
					uek_kernel_devel_install ;;
				5) oracle_pre_kernel_devel
					datto_install ;;
				6) oracle_pre_kernel_devel
					AIPLinux_install ;;
				7) oracle_pre_kernel_devel
					datto_remove ;;
				8) aas ;;
				9) aas_remove ;;
				10) show_help ;;
				11) break ;;
			esac
		else
			number_select "Please select install menu from" "Agent" "GUI" "[1]-[2]All" "Install Kernel-devel" "Install datto driver" "Install AIPLinux" "Uninstall datto driver" "help" "exit"
			case "$?" in
				1) Agent ;;
				2) GUI ;;
				3) All ;;
				4) oracle_pre_kernel_devel
					uek_kernel_devel_install ;;
				5) oracle_pre_kernel_devel
					datto_install ;;
				6) oracle_pre_kernel_devel
					AIPLinux_install ;;
				7) oracle_pre_kernel_devel
					datto_remove ;;
				8) show_help ;;
				9) break ;;
			esac
		fi
	done
}

suse_select_options()
{
	while [ 1 ] ; do
		number_select "Please select install menu from" "Agent" "GUI" "[1]-[2]All" "Install Kernel-devel" "Install datto driver" "Install AIPLinux" "Uninstall datto driver" "help" "exit"
	  	case "$?" in
			1) Agent ;;
			2) GUI ;;
			3) All ;;
			4) suse_pre_kernel_devel
				suse_kernel_devel_install ;;
			5) suse_pre_kernel_devel
				suse_datto_install ;;
			6) suse_pre_kernel_devel
				AIPLinux_install ;;
			7) suse_pre_kernel_devel
				datto_remove ;;
			8) show_help ;;
			9) break ;;
		esac
	done
}

Debian_select_options()
{
	while [ 1 ] ; do
		number_select "Please select install menu from" "Agent" "GUI" "[1]-[2]All" "Install Kernel-devel" "Install datto driver" "Install AIPLinux" "Uninstall datto driver" "help" "exit"
	  	case "$?" in
			1) Agent ;;
			2) GUI ;;
			3) All ;;
			4) Debian_pre_kernel_devel
				Debian_kernel_devel_install ;;
			5) Debian_pre_kernel_devel
				Debian_datto_install ;;
			6) Debian_pre_kernel_devel
				AIPLinux_install ;;
			7) Debian_pre_kernel_devel
				datto_remove ;;
			8) show_help ;;
			9) break ;;
		esac
	done
}

Amazon_select_options()
{
	while [ 1 ] ; do
		number_select "Please select install menu from" "Agent" "GUI" "[1]-[2]All" "Install Kernel-devel" "Install datto driver" "Install AIPLinux" "Uninstall datto driver" "help" "exit"
	  	case "$?" in
			1) Agent ;;
			2) GUI ;;
			3) All ;;
			4) Amazon_pre_kernel_devel
				Amazon_kernel_devel_install ;;
			5) Amazon_pre_kernel_devel
				amazon_datto_install ;;
			6) Amazon_pre_kernel_devel
				AIPLinux_install ;;
			7) Amazon_pre_kernel_devel
				datto_remove ;;
			8) show_help ;;
			9) break ;;
		esac
	done
}

redhat()
{
	if [ "$net_ok" == "0" ]; then
		if [ "$subsc" == "0" ]; then 
			#net ok and subscription ok
			if [ "$OSVER" -ge 6 ]; then
				RHEL6_later_net
			fi
		else
			#net ok and subscription x
			if [ "$OSVER" -le 7 ]; then
				RHEL6_7_nonet
			elif [ "$OSVER" -ge 8 ]; then
				RHEL8_nonet
			fi
		fi
	else
		#net x and subscription x
		if [ "$OSVER" -le 7 ]; then
			RHEL6_7_nonet
		elif [ "$OSVER" -ge 8 ]; then
			RHEL8_nonet
		fi
	fi
}

CentOS()
{
	if [ $net_ok -eq 0 ]; then 
		#net ok 
		if [ "$OSVER" -eq 6 ]; then
			cent6_repo
			Cent6_later_net
			cent6_repo_res
		elif [ "$OSVER" -ge 7 ]; then
			Cent6_later_net
		fi
	else
		#net x 
		if [ "$OSVER" -le 7 ]; then
			Cent6_7_nonet
		elif [ "$OSVER" -ge 8 ]; then
			Cent8_nonet
		fi
	fi
}

Asianux()
{	
	if [ $net_ok -eq 0 ]; then 
		#net ok and subscription ok
		if [ "$OSVER" -ge 7 ]; then
			Cent6_later_net
		fi
	else
		#net x and subscription x
		if [ "$OSVER" -eq 7 ]; then
			Cent6_7_nonet
		elif [ "$OSVER" -ge 8 ]; then
			Cent8_nonet
		fi
	fi
}

Oracle()
{
	if [ $net_ok -eq 0 ]; then 
		#net ok and subscription ok
		if [ "$OSVER" -ge 6 ]; then
			Cent6_later_net
		fi
	else
		#net x and subscription x
		if [ "$OSVER" -le 7 ]; then
			Cent6_7_nonet
		elif [ "$OSVER" -ge 8 ]; then
			Cent8_nonet
		fi
	fi
}

RHELClone()
{
	if [ $net_ok -eq 0 ]; then 
		#net ok and subscription ok
		if [ "$OSVER" -ge 8 ]; then
			Cent6_later_net
		fi
	else
		#net x and subscription x
		if [ "$OSVER" -ge 8 ]; then
			Cent8_nonet
		fi
	fi
}

Amazon()
{	
	if [ $net_ok -eq 0 ]; then 
		#net ok
		case "$OSVER" in
		2)
			Amazon_net
			;;
		esac
	fi
}

openSUSE()
{
	if [ $net_ok -eq 0 ]; then 
		#net ok
		if [ "$OSVER" -ge 15 ]; then
			SUSE_net
		fi
	else
		echo "Please connect internet."
	fi
}

E_SUSE()
{
	if [ $net_ok -eq 0 ]; then 
		#net ok 
		if [ "$OSVER" -ge 15 ]; then
			SUSE_net
		fi
	else
		if [ "$OSVER" -ge 15 ]; then
			SUSE_nonet
		fi
	fi

}

Debian()
{
	if [ $net_ok -eq 0 ]; then 
		#net ok
		if [ "$OSVER" -ge 94 ]; then
			Debian_net
		elif [ "$OSVER" -ge 10 ]; then
			Debian_net
		fi
	else
		if [ "$OSVER" -ge 94 ]; then
			Debian_nonet
		elif [ "$OSVER" -ge 10 ]; then
			Debian_nonet
		fi
	fi
}

Agent()
{	
	print_head

	echo -e "${color_blue}[ Install Agent rpm ]${color_off}"
	echo ""

	unset MISSING
	unset EXIST

Agent=1
if [ "$OSDIST" = "Debian" ] || [ "$OSDIST" = "Ubuntu" ]; then
	for dpkg_e in ${List_Agent}; do
		if dpkg-query -l $dpkg_e 2>/dev/null | grep ii &>/dev/null; then
			EXIST="$dpkg_e $EXIST"
		fi
	done

	echo -e "${color_blue}Already installed rpm packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST ]"${color_off}"
	echo ""

	for dpkg in ${List_Agent}; do
		if ! dpkg-query -l $dpkg 2>/dev/null | grep ii &>/dev/null; then
					MISSING="$dpkg $MISSING"
		fi
	done
else
	for RPM_e in ${List_Agent}; do
		if rpm -q $RPM_e &>/dev/null; then
			if [[ $RPM = *.x86_64 || *.noarch ]]; then
					EXIST="$RPM_e $EXIST"
			fi
		fi
	done

	echo -e "${color_blue}Already installed rpm packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST ]"${color_off}"
	echo ""

	for RPM in ${List_Agent}; do
		if ! rpm -q $RPM &>/dev/null; then
			if [[ $RPM = *.x86_64 || *.noarch ]]; then
				if ! rpm -q ${RPM/%.x86_64/.noarch} &>/dev/null; then
					MISSING="$RPM $MISSING"
				fi
			fi
		fi
	done
fi

	if [ -n "$MISSING" ]; then
		echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $MISSING ]"${color_off}"
		echo ""
		if [ "$OSDIST" = "RHEL" ]; then
			redhat
		elif [ "$OSDIST" = "CentOS" ]; then
			CentOS
		elif [ "$OSDIST" = "Asianux" ]; then
			Asianux
		elif [ "$OSDIST" = "Oracle" ]; then
			Oracle
		elif [ "$OSDIST" = "AlmaLinux" ] || [ "$OSDIST" = "MIRACLE" ] || [ "$OSDIST" = "Rocky" ]; then
			RHELClone
		elif [ "$OSDIST" = "openSUSE" ]; then
			openSUSE
		elif [ "$OSDIST" = "SLED" ] || [ "$OSDIST" = "SLES" ]; then
			E_SUSE
		elif [ "$OSDIST" = "Debian" ] || [ "$OSDIST" = "Ubuntu" ]; then
			Debian
		elif [ "$OSDIST" = "Amazon" ]; then
			Amazon
		fi
	fi
}

GUI()
{
	print_head

	echo -e "${color_blue}[ Install GUI rpm ]${color_off}"
	echo ""
	ohNo
	unset MISSING
	unset EXIST

	Agent=0
if [ "$OSDIST" = "Debian" ] || [ "$OSDIST" = "Ubuntu" ]; then
	for dpkg_e in ${List_GUI}; do
		if dpkg-query -l $dpkg_e 2>/dev/null | grep ii &>/dev/null; then
					EXIST="$dpkg_e $EXIST"
		fi
	done

	echo -e "${color_blue}Already installed rpm packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST ]"${color_off}"
	echo ""

	for dpkg in ${List_GUI}; do
		if ! dpkg-query -l $dpkg 2>/dev/null | grep ii &>/dev/null; then
					MISSING="$dpkg $MISSING"
		fi
	done
else
	for RPM_e in ${List_GUI}; do
		if rpm -q $RPM_e &>/dev/null; then
			if [[ $RPM = *.x86_64 || *.noarch ]]; then
					EXIST="$RPM_e $EXIST"
			fi
		fi
	done

	echo -e "${color_blue}Already installed rpm packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST ]"${color_off}"
	echo ""

	for RPM in ${List_GUI}; do
		if ! rpm -q $RPM &>/dev/null; then
			if [[ $RPM = *.x86_64 || *.noarch ]]; then
				if ! rpm -q ${RPM/%.x86_64/.noarch} &>/dev/null; then
					MISSING="$RPM $MISSING"
				fi
			fi
		fi
	done
fi

	if [ -n "$MISSING" ]; then
		echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $MISSING ]"${color_off}"
		echo ""
		if [ "$OSDIST" = "RHEL" ]; then
			redhat
		elif [ "$OSDIST" = "CentOS" ]; then
			CentOS
		elif [ "$OSDIST" = "Asianux" ]; then
			Asianux
		elif [ "$OSDIST" = "Oracle" ]; then
			Oracle
		elif [ "$OSDIST" = "AlmaLinux" ] || [ "$OSDIST" = "MIRACLE" ] || [ "$OSDIST" = "Rocky" ]; then
			RHELClone
		elif [ "$OSDIST" = "openSUSE" ]; then
			openSUSE
		elif [ "$OSDIST" = "SLED" ] || [ "$OSDIST" = "SLES" ]; then
			E_SUSE
		elif [ "$OSDIST" = "Debian" ] || [ "$OSDIST" = "Ubuntu" ]; then
			Debian
		elif [ "$OSDIST" = "Amazon" ]; then
			Amazon
		fi
	fi
}

All()
{
	print_head

	echo -e "${color_blue}[ Install All packages ]${color_off}"
	echo ""
	ohNo
	unset MISSING
	unset EXIST

	Agent=0
if [ "$OSDIST" = "Debian" ] || [ "$OSDIST" = "Ubuntu" ]; then
	for dpkg_e in ${List_All}; do
		if dpkg-query -l $dpkg_e 2>/dev/null | grep ii &>/dev/null; then
			EXIST="$dpkg_e $EXIST"
		fi
	done

	echo -e "${color_blue}Already installed packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST ]"${color_off}"
	echo ""

	for dpkg in ${List_All}; do
		if ! dpkg-query -l $dpkg 2>/dev/null | grep ii &>/dev/null; then
					MISSING="$dpkg $MISSING"
		fi
	done
else
	for RPM_e in ${List_All}; do
		if rpm -q $RPM_e &>/dev/null; then
			if [[ $RPM = *.x86_64 || *.noarch ]]; then
					EXIST="$RPM_e $EXIST"
			fi
		fi
	done

	echo -e "${color_blue}Already installed packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST ]"${color_off}"
	echo ""

	for RPM in ${List_All}; do
		if ! rpm -q $RPM &>/dev/null; then
			if [[ $RPM = *.x86_64 || *.noarch ]]; then
				if ! rpm -q ${RPM/%.x86_64/.noarch} &>/dev/null; then
					MISSING="$RPM $MISSING"
				fi
			fi
		fi
	done
fi
	if [ -n "$MISSING" ]; then
		echo -e "${color_blue}The following packages are not installed:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $MISSING ]"${color_off}"
		echo ""
		if [ "$OSDIST" = "RHEL" ]; then
			redhat
		elif [ "$OSDIST" = "CentOS" ]; then
			CentOS
		elif [ "$OSDIST" = "Asianux" ]; then
			Asianux
		elif [ "$OSDIST" = "Oracle" ]; then
			Oracle
		elif [ "$OSDIST" = "AlmaLinux" ] || [ "$OSDIST" = "MIRACLE" ] || [ "$OSDIST" = "Rocky" ]; then
			RHELClone
		elif [ "$OSDIST" = "openSUSE" ]; then
			openSUSE
		elif [ "$OSDIST" = "SLED" ] || [ "$OSDIST" = "SLES" ]; then
			E_SUSE
		elif [ "$OSDIST" = "Debian" ] || [ "$OSDIST" = "Ubuntu" ]; then
			Debian
		elif [ "$OSDIST" = "Amazon" ]; then
			Amazon
		fi
	fi
}

##################################################
pre_kernel_devel(){

List1="kernel-devel-$(uname -r)"
List2="gcc.x86_64 make.x86_64 perl.x86_64"
NotNeedList="datto-el-rpm-release"
List_datto="dkms dkms-dattobd libdattobd dattobd-utils"
os_ver=`cat /etc/redhat-release | grep -o '[0-9]\.[0-9]*'`
plat=`uname -i`
ker_num=`uname -r | sed -e 's/\(^[0-9]\+\.[0-9]\+\.[0-9]\+\-[0-9]\+\).*$/\1/'`
if [ ${OSVER} -lt 8 ]; then
	release_ver1=`echo "$ker_list" | grep "${ker_num}" | awk -F" " '{print $1 }'`
	release_ver=`echo "$ker_list" | grep "${ker_num}" | awk -F" " '{print $2 }'`
fi
if [ ${OSVER} -le 6 ]; then
	release_ver=`echo "$ker_list" | grep "${ker_num}" | awk -F" " '{print $1 }'`
fi

unset MISSING1
unset MISSING2
unset EXIST1
unset MISSING_datto
unset EXIST_datto
unset NotNeed

for RPM_e in $List1; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				EXIST1="$RPM_e $EXIST1"
		fi
	fi
done

for RPM_e in $List2; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				EXIST2="$RPM_e $EXIST2"
		fi
	fi
done

for RPM_e in $List_datto; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				EXIST_datto="$RPM_e $EXIST_datto"
		fi
	fi
done

for RPM_e in $NotNeedList; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				NotNeed="$RPM_e $NotNeed"
		fi
	fi
done

for RPM in $List1; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.i686 ]]; then
			if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
				MISSING1="$RPM $MISSING1"
			fi
		fi
	fi
done

for RPM in $List2; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.i686 ]]; then
			if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
				MISSING2="$RPM $MISSING2"
			fi
		fi
	fi
done

for RPM in $List_datto; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.i686 ]]; then
			if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
				MISSING_datto="$RPM $MISSING_datto"
			fi
		fi
	fi
done

}

refer_DVD(){
	ins_check_devel=`rpm -qa kernel-devel | wc -l`
	if [ $ins_check_devel = 0 ]; then
		echo -e "${color_blue}kernel-devel is not installed.${color_off}"
		echo "=== Internet connection or subscription failed. Please insert the OS DVD."
		echo -n "Input DVD mount path [e.g.: /run/media/root/CentOS 7 x86_64] :"
		read dvd_path
		dvd_path=`echo "$dvd_path" | sed -e 's/ /%20/g'`
		if [ ${OSVER} -eq 6 -o ${OSVER} -eq 7 ]; then
			$(repo)
		elif [ ${OSVER} -ge 8 ]; then
			$(repo8)
		fi
		echo ""
		echo -e "${color_blue}Created yum repository. Press any key to continue...${color_off}"	
		echo ""
		echo -e "${color_blue}Install the required module using yum command.${color_off}"	
		echo "kernel-devel install"
		if [ ${OSVER} -ge 8 ]; then
			$inst_com --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck install $List1
		else
			$inst_com --disablerepo=* --enablerepo=DVD --nogpgcheck install $List1
		fi
		if [ $? -eq 1 ]; then
			echo -e "${color_red}yum install is failed.${color_off}"
			echo -e "${color_red}Please get $List1 in another way.${color_off}"
			echo -e "${color_red}Please install $List1.${color_off}"
			echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
			DVD_remove							
			exit 1
		fi
	fi
}

kernel_devel_install() {

print_head

echo -e "${color_blue}[ Install kernel-devel ]${color_off}"

if [ -n "$MISSING1" ]; then
	echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $MISSING1 ]"${color_off}"
	echo ""

	vault_remove
	DVD_remove

	if [ $net_ok -eq 0 ]; then 	#つながる
		echo "=== Internet connection OK."
		if [ "$OSDIST" = "RHEL" ]; then
			if [ $subsc -eq 0 ]; then
				echo "=== Registerd subscription OK."
			else
				echo "=== Not registerd subscription."
				refer_DVD
			fi
		fi
		echo "yum clean all"
		yum clean all
		if [ "$OSDIST" = "RHEL" ]; then
			vault_remove
			ol_remove
		fi
		ins_check_devel=`rpm -qa kernel-devel | wc -l`
		if [ $ins_check_devel = 0 ]; then
			if [ "${os_ver}" != "${release_ver}"  ]; then		
				ker_dev_not_inst_not_match
			fi
			if [ "${os_ver}" = "${release_ver}"  ]; then
				ker_dev_not_inst_match
			fi
		else
			if [ "${os_ver}" != "${release_ver}"  ]; then
				ker_dev_inst_not_match
			fi
			if [ "${os_ver}" = "${release_ver}"  ]; then
				ker_dev_inst_match
			fi
		fi
	else		##########################yum がつながらない　DVD参照
		refer_DVD
	fi
else
	echo ""
	echo -e "${color_blue}Already installed rpm packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST1 ]"${color_off}"
	echo ""
fi
DVD_remove
}

oracle_pre_kernel_devel(){

chk_uek=`uname -r | grep uek | wc -l`
if [ $chk_uek -eq 1 ]; then
	List1="kernel-uek-devel-$(uname -r)"
else
	List1="kernel-devel-$(uname -r)"
fi

List2="gcc.x86_64 make.x86_64 perl.x86_64"
NotNeedList="datto-el-rpm-release"
List_datto="dkms dkms-dattobd libdattobd dattobd-utils"
os_ver=`cat /etc/redhat-release | grep -o '[0-9]\.[0-9]*'`
plat=`uname -i`
ker_num=`uname -r | sed -e 's/\(^[0-9]\+\.[0-9]\+\.[0-9]\+\-[0-9]\+\).*$/\1/'`

if [ ${OSVER} -ge 7 ]; then
	release_ver1=`echo "$ker_list" | grep "${ker_num}" | awk -F" " '{print $1 }'`
	release_ver=`echo "$ker_list" | grep "${ker_num}" | awk -F" " '{print $2 }'`
fi
if [ ${OSVER} -le 6 ]; then
	release_ver=`echo "$ker_list" | grep "${ker_num}" | awk -F" " '{print $1 }'`
fi

unset MISSING1
unset MISSING2
unset EXIST1
unset MISSING_datto
unset EXIST_datto
unset NotNeed

for RPM_e in $List1; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				EXIST1="$RPM_e $EXIST1"
		fi
	fi
done

for RPM_e in $List2; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				EXIST2="$RPM_e $EXIST2"
		fi
	fi
done

for RPM_e in $List_datto; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				EXIST_datto="$RPM_e $EXIST_datto"
		fi
	fi
done

for RPM_e in $NotNeedList; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				NotNeed="$RPM_e $NotNeed"
		fi
	fi
done

for RPM in $List1; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.i686 ]]; then
			if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
				MISSING1="$RPM $MISSING1"
			fi
		fi
	fi
done

for RPM in $List2; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.i686 ]]; then
			if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
				MISSING2="$RPM $MISSING2"
			fi
		fi
	fi
done

for RPM in $List_datto; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.i686 ]]; then
			if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
				MISSING_datto="$RPM $MISSING_datto"
			fi
		fi
	fi
done

}

uek_kernel_devel_install() {

print_head

echo -e "${color_blue}[ Install kernel-devel ]${color_off}"

if [ -n "$MISSING1" ]; then
	echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $MISSING1 ]"${color_off}"
	echo ""
	vault_remove
	DVD_remove

	if [ $net_ok -eq 0 ]; then 	#つながる場合
		echo "=== Internet connection OK."
		echo "yum clean all"
		yum clean all
		oracle_repo
		yum -y install $MISSING1
	else		##########################yum がつながらない　DVD参照
		ins_check_devel=`rpm -qa | grep kernel-uek-devel | wc -l`
		if [ $ins_check_devel -eq 0 ]; then
			echo "=== Internet connection failed. Please insert the OS DVD."
			echo -n "Input DVD mount path [e.g.: /run/media/root/CentOS 7 x86_64] :"
			read dvd_path
			dvd_path=`echo "$dvd_path" | sed -e 's/ /%20/g'`
			if [ ${OSVER} = 6 -o ${OSVER} = 7 ]; then
				$(repo)
			elif [ ${OSVER} = 8 ]; then
				$(repo8)
			fi
			echo ""
			echo -e "${color_blue}Created yum repository. Press any key to continue...${color_off}"	
			echo ""
			if [ ${OSVER} -ge 6 ]; then
				echo ""
				echo -e "${color_blue}Install the required module using yum command.${color_off}"	
				echo "kernel-devel install"
				if [ ${OSVER} -eq 8 ]; then
					$inst_com --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck install $List1
				else
					$inst_com --disablerepo=* --enablerepo=DVD --nogpgcheck install $List1
				fi
				if [ $? = 1 ]; then
					echo -e "${color_red}yum install is failed.${color_off}"
					echo -e "${color_red}Please get $List1 in another way.${color_off}"
					echo -e "${color_red}Please install $List1.${color_off}"
					echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
					DVD_remove							
					exit 1
				fi
			fi
		fi
	fi
else
	echo ""
	echo -e "${color_blue}Already installed rpm packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST1 ]"${color_off}"
	echo ""
fi
DVD_remove
}


suse_pre_kernel_devel(){

ker=`uname -r | sed -e s'/-[a-z]*$//'`
kernel_num=`rpm -qa | grep $ker | grep -E kernel-default-[0-9] | sed -e s'/[a-z]*-[a-z]*-//' | sed -e s'/.x86_64//'`
List1="kernel-devel-$kernel_num"
List2="gcc.x86_64 make.x86_64 perl.x86_64"
knum=`echo $List1 | grep $ker | sed -e s'/kernel-devel-//'| sed -e s'/.x86_64//'`

List_datto="dkms dkms-dattobd libdattobd1 dattobd-utils"

unset MISSING1
unset MISSING2
unset EXIST1
unset MISSING_datto
unset EXIST_datto

for RPM_e in $List1; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				EXIST1="$RPM_e $EXIST1"
		fi
	fi
done

for RPM_e in $List2; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				EXIST2="$RPM_e $EXIST2"
		fi
	fi
done

for RPM_e in $List_datto; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				EXIST_datto="$RPM_e $EXIST_datto"
		fi
	fi
done

for RPM in $List1; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.noarch ]]; then
			if ! rpm -q ${RPM/%.x86_64/.noarch} &>/dev/null; then
				MISSING1="$RPM $MISSING1"
			fi
		fi
	fi
done

for RPM in $List2; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.i686 ]]; then
			if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
				MISSING2="$RPM $MISSING2"
			fi
		fi
	fi
done

for RPM in $List_datto; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.i686 ]]; then
			if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
				MISSING_datto="$RPM $MISSING_datto"
			fi
		fi
	fi
done

}

suse_kernel_devel_install() {

print_head

echo -e "${color_blue}[ Install kernel-devel ]${color_off}"
open_dl_list="kernel-macros-$knum kernel-devel-$knum kernel-syms-$knum"
find_path="/var/cache/zypp/packages"

if [ -n "$MISSING1" ]; then
	echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $MISSING1 ]"${color_off}"
	echo ""
	devel=`echo $MISSING1 | sed -e s'/.x86_64//'`
	if [ "$net_ok" -eq "0" ]; then 	#つながる場合
		echo "=== Internet connection OK."
		ins_check_devel=`rpm -qa kernel-devel | wc -l`
		if [ $ins_check_devel = 0 ]; then
			if [ "$OSDIST" = "SLES" ] || [ "$OSDIST" = "SLED" ]; then	
				kver=`uname -r | sed -e s'/-[a-z].*//'`
				zypper --non-interactive install -C "kernel-syms = ${kver}"
			elif [ "$OSDIST" = "openSUSE" ]; then
				zypper download $open_dl_list
				rpms=`find $find_path -name kernel-macros-$knum* -or -name kernel-devel-$knum* | wc -l` 
				find $find_path -name kernel-macros-$knum* -or -name kernel-devel-$knum* >/tmp/rpms.txt
				if [ $rpms -eq 2 ]; then
					cat /tmp/rpms.txt | while read LINE; do
					rpm -ivh $LINE
					if [ $? -ne 0 ]; then
						echo "Failed to install kernel packages."
					else
						echo "Successful install kernel packages."
						zypper --non-interactive install -C "kernel-syms = $knum"
					fi
					done
				else
					echo "Failed to download packages."
				fi
			fi
		fi
	else
		repo_exist=`zypper repos -E | grep Yes | wc -l`
		if [ $repo_exist -eq 0 ]; then
			echo "=== Internet connection failed and not exist repository. Please insert the OS DVD."
			echo -n "Input DVD mount path [e.g.: /run/media/root/SLE-15-SP2-Full-x86_64209.21.001] :"
			read dvd_path
			if [ ! -e $dvd_path ]; then
				echo "Path not found."
			fi
			suse_repo
			echo -e "${color_blue}Install the required rpm packages using zypper command.${color_off}"
			$inst_com install $MISSING1
			ins_check_devel=`rpm -qa kernel-devel | wc -l`
			if [ $ins_check_devel = 0 ]; then
				if [ "$OSDIST" = "SLES" ] || [ "$OSDIST" = "SLED" ]; then	
					kver=`uname -r | sed -e s'/-[a-z].*//'`
					zypper --non-interactive install -C "kernel-syms = ${kver}"
				elif [ "$OSDIST" = "openSUSE" ]; then
					zypper download $open_dl_list
					rpms=`find $find_path -name kernel-macros-$knum* -or -name kernel-devel-$knum* | wc -l` 
					find $find_path -name kernel-macros-$knum* -or -name kernel-devel-$knum* >/tmp/rpms.txt
					if [ $rpms -eq 2 ]; then
						cat /tmp/rpms.txt | while read LINE; do
						rpm -ivh $LINE
						if [ $? -ne 0 ]; then
							echo "Failed to install kernel packages."
						else
							echo "Successful install kernel packages."
							zypper --non-interactive install -C "kernel-syms = $knum"
						fi
						done
					else
						echo "Failed to download packages."
					fi
				fi
			fi
		else
			echo "=== Internet connection failed."
			echo -e "${color_blue}Install the required rpm packages using zypper command.${color_off}"
			$inst_com install $MISSING1
			ins_check_devel=`rpm -qa kernel-devel | wc -l`
			if [ $ins_check_devel = 0 ]; then
				if [ "$OSDIST" = "SLES" ] || [ "$OSDIST" = "SLED" ]; then	
					kver=`uname -r | sed -e s'/-[a-z].*//'`
					zypper --non-interactive install -C "kernel-syms = ${kver}"

				elif [ "$OSDIST" = "openSUSE" ]; then
					zypper download $open_dl_list
					rpms=`find $find_path -name kernel-macros-$knum* -or -name kernel-devel-$knum* | wc -l` 
					find $find_path -name kernel-macros-$knum* -or -name kernel-devel-$knum* >/tmp/rpms.txt
					if [ $rpms -eq 2 ]; then
						cat /tmp/rpms.txt | while read LINE; do
						rpm -ivh $LINE
						if [ $? -ne 0 ]; then
							echo "Failed to install kernel packages."
						else
							echo "Successful install kernel packages."
							zypper --non-interactive install -C "kernel-syms = $knum"
						fi
						done
					else
						echo "Failed to download packages."
					fi
				fi
			fi
		fi
	fi
else
	echo ""
	echo -e "${color_blue}Already installed rpm packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST1 ]"${color_off}"
	echo ""
fi
}

Debian_pre_kernel_devel(){

List1="linux-headers-$(uname -r)"
List2="gcc make perl"
List_datto="dkms dattobd-dkms libdattobd1 dattobd-utils"

unset MISSING1
unset MISSING2
unset EXIST1
unset EXIST2
unset MISSING_datto
unset EXIST_datto

for ker in $List1; do
	if dpkg-query -l $ker 2>/dev/null | grep ii &>/dev/null; then
		EXIST1="$ker $EXIST1"
	fi
done

for dpkg_e in $List2; do
	if dpkg-query -l $dpkg_e 2>/dev/null | grep ii &>/dev/null; then
			EXIST2="$dpkg_e $EXIST2"
	fi
done

for pdatto in $List_datto; do
	if dpkg-query -l $pdatto 2>/dev/null | grep ii &>/dev/null; then
			EXIST_datto="$pdatto $EXIST_datto"
	fi
done

for ker in $List1; do
	if ! dpkg-query -l $ker 2>/dev/null | grep ii &>/dev/null; then
		MISSING1="$ker $MISSING1"
	fi
done

for dpkg_e in $List2; do
	if ! dpkg-query -l $dpkg_e 2>/dev/null | grep ii &>/dev/null; then
			MISSING2="$dpkg_e $MISSING2"
	fi
done

for pdatto in $List_datto; do
	if ! dpkg-query -l $pdatto 2>/dev/null | grep ii &>/dev/null; then
			MISSING_datto="$pdatto $MISSING_datto"
	fi
done

}

Debian_kernel_devel_install() {

print_head

echo -e "${color_blue}[ Install linux-headers ]${color_off}"

if [ -n "$MISSING1" ]; then
	echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $MISSING1 ]"${color_off}"
	echo ""

	ins_check_devel=`dpkg-query -l linux-headers-$(uname -r) 2>/dev/null | grep ii | wc -l`
	if [ "$net_ok" -eq "0" ]; then 	#つながる場合
		echo "=== Internet connection OK."
		if [ $ins_check_devel = 0 ]; then
			$inst_com linux-headers-$(uname -r) 2>/dev/null
			after_check=`dpkg-query -l linux-headers-$(uname -r) 2>/dev/null | grep ii | wc -l`
			if [ $after_check -eq 1 ]; then
				echo "Successful install kernel packages."
			else
				echo "Failed to install kernel packages."
			fi
		fi
	else	#not connect internet
		if [ $recreate_cdrom -eq 0 ]; then
			echo "=== Internet connection failed. Please insert the OS DVD."
			echo -n "Input DVD mount path [e.g.: /media/cdrom0] :"
			read dvd_path
			if [ ! -e $dvd_path ]; then
				echo "Path not found."
			fi
			apt-cdrom add
			apt-cdrom ident
			echo "Run apt update....."
			apt update -y >/dev/null 2>&1
			if [ $ins_check_devel = 0 ]; then
				$inst_com linux-headers-$(uname -r) 2>/dev/null
				after_check=`dpkg-query -l linux-headers-$(uname -r) 2>/dev/null | grep ii | wc -l`
				if [ $after_check -eq 1 ]; then
					echo "Successful install kernel packages."
				else
					echo "Failed to install kernel packages."
				fi
			fi
		elif [ $recreate_cdrom -eq 1 ]; then
			echo "Enable cdrom repository."
			sed -i 's/^# deb cdrom/\deb cdrom/' $apt_src
			apt-cdrom ident
			echo "Run apt update....."
			apt update -y >/dev/null 2>&1
			if [ $ins_check_devel = 0 ]; then
				$inst_com linux-headers-$(uname -r) 2>/dev/null
				after_check=`dpkg-query -l linux-headers-$(uname -r) 2>/dev/null | grep ii | wc -l`
				if [ $after_check -eq 1 ]; then
					echo "Successful install kernel packages."
				else
					echo "Failed to install kernel packages."
				fi
			fi
		fi
	fi
else
	echo ""
	echo -e "${color_blue}Already installed rpm packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST1 ]"${color_off}"
	echo ""
fi
}

Amazon_pre_kernel_devel(){

List1="kernel-devel-$(uname -r)"
List2="gcc.x86_64 make.x86_64 perl.x86_64"
List_datto="dkms dkms-dattobd libdattobd dattobd-utils"


unset MISSING1
unset MISSING2
unset EXIST1
unset EXIST2
unset MISSING_datto
unset EXIST_datto


for RPM_e in $List1; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				EXIST1="$RPM_e $EXIST1"
		fi
	fi
done

for RPM_e in $List2; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				EXIST2="$RPM_e $EXIST2"
		fi
	fi
done

for RPM_e in $List_datto; do
	if rpm -q $RPM_e &>/dev/null; then
			EXIST_datto="$RPM_e $EXIST_datto"
	fi
done

for RPM in $List1; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.i686 ]]; then
			if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
				MISSING1="$RPM $MISSING1"
			fi
		fi
	fi
done

for RPM in $List2; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.i686 ]]; then
			if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
				MISSING2="$RPM $MISSING2"
			fi
		fi
	fi
done

for RPM in $List_datto; do
	if ! rpm -q $RPM &>/dev/null; then
			MISSING_datto="$RPM $MISSING_datto"
	fi
done

}

Amazon_kernel_devel_install() {

print_head

echo -e "${color_blue}[ Install kernel-devel ]${color_off}"

if [ -n "$MISSING1" ]; then
	echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $MISSING1 ]"${color_off}"
	echo ""

	if [ $net_ok -eq 0 ]; then 	#つながる場合
		echo "=== Internet connection OK."
		echo "yum clean all"
		yum clean all
		$inst_com install $MISSING1
			if [ $? -eq 1 ]; then
				echo -e "${color_red}yum install is failed.${color_off}"
				echo -e "${color_red}Please get "$MISSING1" in another way.${color_off}"
				echo -e "${color_red}Please install "$MISSING1".${color_off}"
				echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
				exit 1
			fi
	fi
else
	echo ""
	echo -e "${color_blue}Already installed rpm packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST1 ]"${color_off}"
	echo ""
fi

}

#################datto section######################
datto_rpm() {
	pkg_dkms=`ls $dir/packages | grep dkms | grep el$OSVER | head -1`
	echo "[ rpm -Uvh $pkg_dkms ]"
	rpm -Uvh --nodeps $dir/packages/$pkg_dkms
	pkg_dkms_dattobd=`ls $dir/packages/datto | grep dkms-dattobd | grep el$OSVER | head -1`
	echo "[ rpm -Uvh --force $pkg_dkms_dattobd ]"
	rpm -Uvh --force $dir/packages/datto/$pkg_dkms_dattobd
	pkg_libdattobd=`ls $dir/packages/datto | grep libdattobd | grep el$OSVER | head -1`
	echo "[ rpm -Uvh --force $pkg_libdattobd ]"
	rpm -Uvh --force $dir/packages/datto/$pkg_libdattobd
	pkg_datto_utils=`ls $dir/packages/datto | grep dattobd-utils | grep el$OSVER | head -1`
	echo "[ rpm -Uvh --force $pkg_datto_utils ]"
	rpm -Uvh --force $dir/packages/datto/$pkg_datto_utils
}

suse_datto_rpm() {
	pkg_dkms=`ls $dir/packages | grep dkms | grep bp15 | head -1`
	echo "[ rpm -Uvh $pkg_dkms ]"		
	rpm -Uvh --nodeps $dir/packages/$pkg_dkms
	pkg_dkms_dattobd=`ls $dir/packages/datto | grep dkms-dattobd | grep sle$OSVER | head -1`
	echo "[ rpm -Uvh --force $pkg_dkms_dattobd ]"
	rpm -Uvh --force $dir/packages/datto/$pkg_dkms_dattobd
	pkg_libdattobd=`ls $dir/packages/datto | grep libdattobd | grep sle$OSVER | head -1`
	echo "[ rpm -Uvh --force $pkg_libdattobd ]"
	rpm -Uvh --force $dir/packages/datto/$pkg_libdattobd
	pkg_datto_utils=`ls $dir/packages/datto | grep dattobd-utils | grep sle$OSVER | head -1`
	echo "[ rpm -Uvh --force $pkg_datto_utils ]"
	rpm -Uvh --force $dir/packages/datto/$pkg_datto_utils
}

Debian_datto_dpkg() {
if [ "$OSDIST" = "Debian" ]; then
	if [ $OSVER -eq 94 ]; then
		pkg_dkms=`ls $dir/packages | grep dkms | grep deb | head -1`
		echo "[ apt -y --reinstall install $pkg_dkms ]"		
		apt -y install $dir/packages/$pkg_dkms
		pkg_dkms_dattobd=`ls $dir/packages/datto | grep dattobd-dkms | grep debian9 | head -1`
		echo "[ apt -y --reinstall install $pkg_dkms_dattobd ]"
		apt -y --reinstall install $dir/packages/datto/$pkg_dkms_dattobd
		pkg_libdattobd=`ls $dir/packages/datto | grep libdattobd | grep debian9 | head -1`
		echo "[ apt -y --reinstall install $pkg_libdattobd ]"
		apt -y --reinstall install $dir/packages/datto/$pkg_libdattobd
		pkg_datto_utils=`ls $dir/packages/datto | grep dattobd-utils | grep debian9 | head -1`
		echo "[ apt -y --reinstall install $pkg_datto_utils ]"
		apt -y --reinstall install $dir/packages/datto/$pkg_datto_utils
	else
		pkg_dkms=`ls $dir/packages | grep dkms | grep deb | head -1`
		echo "[ apt -y --reinstall install $pkg_dkms ]"		
		apt -y install $dir/packages/$pkg_dkms
		pkg_dkms_dattobd=`ls $dir/packages/datto | grep dattobd-dkms | grep debian$OSVER | head -1`
		echo "[ apt -y --reinstall install $pkg_dkms_dattobd ]"
		apt -y --reinstall install $dir/packages/datto/$pkg_dkms_dattobd
		pkg_libdattobd=`ls $dir/packages/datto | grep libdattobd | grep debian$OSVER | head -1`
		echo "[ apt -y --reinstall install $pkg_libdattobd ]"
		apt -y --reinstall install $dir/packages/datto/$pkg_libdattobd
		pkg_datto_utils=`ls $dir/packages/datto | grep dattobd-utils | grep debian$OSVER | head -1`
		echo "[ apt -y --reinstall install $pkg_datto_utils ]"
		apt -y --reinstall install $dir/packages/datto/$pkg_datto_utils
	fi
elif [ "$OSDIST" = "Ubuntu" ]; then
	pkg_dkms=`ls $dir/packages | grep dkms | grep deb | head -1`
	echo "[ apt -y --reinstall install $pkg_dkms ]"		
	apt -y install $dir/packages/$pkg_dkms
	pkg_dkms_dattobd=`ls $dir/packages/datto | grep dattobd-dkms | grep ubuntu$OSVER | head -1`
	echo "[ apt -y --reinstall install $pkg_dkms_dattobd ]"
	apt -y --reinstall install $dir/packages/datto/$pkg_dkms_dattobd
	pkg_libdattobd=`ls $dir/packages/datto | grep libdattobd | grep ubuntu$OSVER | head -1`
	echo "[ apt -y --reinstall install $pkg_libdattobd ]"
	apt -y --reinstall install $dir/packages/datto/$pkg_libdattobd
	pkg_datto_utils=`ls $dir/packages/datto | grep dattobd-utils | grep ubuntu$OSVER | head -1`
	echo "[ apt -y --reinstall install $pkg_datto_utils ]"
	apt -y --reinstall install $dir/packages/datto/$pkg_datto_utils
fi

}

amazon_datto_rpm() {
	if [ ${OSVER} -eq 2 ]; then
		pkg_dkms=`ls $dir/packages | grep dkms | grep el7 | head -1`
		echo "[ rpm -Uvh $pkg_dkms ]"		
		rpm -Uvh --nodeps $dir/packages/$pkg_dkms
		pkg_dkms_dattobd=`ls $dir/packages/datto | grep dkms-dattobd | grep el7 | head -1`
		echo "[ rpm -Uvh --force $pkg_dkms_dattobd ]"
		rpm -Uvh --force $dir/packages/datto/$pkg_dkms_dattobd
		pkg_libdattobd=`ls $dir/packages/datto | grep libdattobd | grep el7 | head -1`
		echo "[ rpm -Uvh --force $pkg_libdattobd ]"
		rpm -Uvh --force $dir/packages/datto/$pkg_libdattobd
		pkg_datto_utils=`ls $dir/packages/datto | grep dattobd-utils | grep el7 | head -1`
		echo "[ rpm -Uvh --force $pkg_datto_utils ]"
		rpm -Uvh --force $dir/packages/datto/$pkg_datto_utils
	fi
}

req_datto_rpm() {

if [ -n "$MISSING1" -o -n "$MISSING2" ]; then
	echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $MISSING1 $MISSING2 ]"${color_off}"
	echo ""
	echo -e "${color_red}There are no required packages. ${color_off}"
	if [ -n "$MISSING1" -a -n "$MISSING2" ]; then
		echo -e "${color_red}Please run the [[1] or [3]] and [4] first. ${color_off}"	
	elif [ -n "$MISSING2" ]; then
		echo -e "${color_red}Please run the [[1] or [3]] first. ${color_off}"
	elif [ -n "$MISSING1" ]; then
		echo -e "${color_red}Please run the [4] first. ${color_off}"
	fi
	req_datto=1
else
	req_datto=0
fi

}

req_datto_dpkg() {

if [ -n "$MISSING1" -o -n "$MISSING2" ]; then
	echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $MISSING1 $MISSING2 ]"${color_off}"
	echo ""
	echo -e "${color_red}There are no required packages. ${color_off}"
	if [ -n "$MISSING1" -a -n "$MISSING2" ]; then
		echo -e "${color_red}Please run the [[1] or [3]] and [4] first. ${color_off}"	
	elif [ -n "$MISSING2" ]; then
		echo -e "${color_red}Please run the [[1] or [3]] first. ${color_off}"
	elif [ -n "$MISSING1" ]; then
		echo -e "${color_red}Please run the [4] first. ${color_off}"
	fi
	req_datto=1
else
	req_datto=0
fi

}

req_AIP_rpm() {

unset MISSING_Agent
unset MISSING_GUI

for RPM in ${List_Agent}; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.noarch ]]; then
			if ! rpm -q ${RPM/%.x86_64/.noarch} &>/dev/null; then
				MISSING_Agent="$RPM $MISSING_Agent"
			fi
		fi
	fi
done

for RPM in ${List_GUI}; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.noarch ]]; then
			if ! rpm -q ${RPM/%.x86_64/.noarch} &>/dev/null; then
				MISSING_GUI="$RPM $MISSING_GUI"
			fi
		fi
	fi
done

if [ -n "$MISSING_Agent" -o -n "$MISSING_GUI" -o -n "$MISSING1" ]; then
	echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $MISSING_Agent ":" $MISSING_GUI ":" $MISSING1 ]"${color_off}"
	echo ""
	echo -e "${color_red}There are no required packages. ${color_off}"
	if [ -n "$MISSING_Agent" -a -n "$MISSING_GUI" -a -n "$MISSING1" ]; then
		echo -e "${color_red}Please run the [[1] or [3]] and [4] first. ${color_off}"
		req_AIP=1
	elif [ -n "$MISSING_Agent" -a -n "$MISSING_GUI" ]; then
		echo -e "${color_red}Please run the [[1] and [3]] first. ${color_off}"
		req_AIP=1
	elif [ -n "$MISSING_Agent" -a -n "$MISSING1" ]; then
		echo -e "${color_red}Please run the [[1] and [4]] first. ${color_off}"
		req_AIP=1
	elif [ -n "$MISSING_GUI" -a -n "$MISSING1" ]; then
		echo -e "${color_red}Please run the [[2] and [4]] first. ${color_off}"
		req_AIP=1
	elif [ -n "$MISSING_Agent" ]; then
		echo -e "${color_red}Please run the [[1] or [3]] first. ${color_off}"
		req_AIP=1
	elif [ -n "$MISSING_GUI" ]; then
		echo -e "${color_red}GUI packages does not installed. ${color_off}"
		echo -e "${color_red}Please run the [2] first. ${color_off}"
		ask_YN "Do you want to continue install AIPLinux?" "Y"
		if [ "$RC_ask_YN" = "Y" ]; then
			req_AIP=0
		else
			req_AIP=1
		fi
	elif [ -n "$MISSING1" ]; then
		echo -e "${color_red}Please run the [4] first. ${color_off}"
		req_AIP=1
	fi
else
	req_AIP=0
fi

}

req_AIP_dpkg() {

unset MISSING_Agent
unset MISSING_GUI

	for dpkg in ${List_Agent}; do
		if ! dpkg-query -l $dpkg 2>/dev/null | grep ii &>/dev/null; then
					MISSING_Agent="$dpkg $MISSING_Agent"
		fi
	done

	for dpkg in ${List_GUI}; do
		if ! dpkg-query -l $dpkg 2>/dev/null | grep ii &>/dev/null; then
					MISSING_GUI="$dpkg $MISSING_GUI"
		fi
	done

if [ -n "$MISSING_Agent" -o -n "$MISSING_GUI" -o -n "$MISSING1" ]; then
	echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $MISSING_Agent ":" $MISSING_GUI ":" $MISSING1 ]"${color_off}"
	echo ""
	echo -e "${color_red}There are no required packages. ${color_off}"
	if [ -n "$MISSING_Agent" -a -n "$MISSING_GUI" -a -n "$MISSING1" ]; then
		echo -e "${color_red}Please run the [[1] or [3]] and [4] first. ${color_off}"
		req_AIP=1
	elif [ -n "$MISSING_Agent" -a -n "$MISSING_GUI" ]; then
		echo -e "${color_red}Please run the [[1] and [3]] first. ${color_off}"
		req_AIP=1
	elif [ -n "$MISSING_Agent" -a -n "$MISSING1" ]; then
		echo -e "${color_red}Please run the [[1] and [4]] first. ${color_off}"
		req_AIP=1
	elif [ -n "$MISSING_GUI" -a -n "$MISSING1" ]; then
		echo -e "${color_red}Please run the [[2] and [4]] first. ${color_off}"
		req_AIP=1
	elif [ -n "$MISSING_Agent" ]; then
		echo -e "${color_red}Please run the [[1] or [3]] first. ${color_off}"
		req_AIP=1
	elif [ -n "$MISSING_GUI" ]; then
		echo -e "${color_red}GUI packages does not installed. ${color_off}"
		echo -e "${color_red}Please run the [2] first. ${color_off}"
		ask_YN "Do you want to continue install AIPLinux?" "Y"
		if [ "$RC_ask_YN" = "Y" ]; then
			req_AIP=0
		else
			req_AIP=1
		fi
	elif [ -n "$MISSING1" ]; then
		echo -e "${color_red}Please run the [4] first. ${color_off}"
		req_AIP=1
	fi
else
	req_AIP=0
fi

}

datto_install() {
print_head

Chk_dracut_ver

echo -e "${color_blue}[ datto driver install ]${color_off}"
req_datto_rpm
if [ ${req_datto} -eq 0 ]; then
	if [ -n "$MISSING_datto" ]; then
		echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $MISSING_datto ]"${color_off}"
		echo ""

		if [ -e $dir/packages -a -e $dir/packages/datto ]; then
			datto_rpm
		else	
			echo "datto package is not exist."
			exit
		fi

		modinfo dattobd > /dev/null 2>&1
		if [ $? -eq 0 ]; then
			echo -e "${color_blue}dattobd Driver install successful.${color_off}"	
		else
			echo -e "${color_red}dattobd Driver install failed.${color_off}"
		fi
		
		datto_path=`modinfo -F filename dattobd`
		insmod $datto_path > /dev/null 2>&1 || true

		echo ""
		echo "[ /proc/datto-info ]"
		cat /proc/datto-info

		chk_dbdctl=`dbdctl | grep "dbdctl help"| wc -l`
		if [ $chk_dbdctl -eq 1 ]; then
			echo -e "${color_blue}datto utils install successful.${color_off}" 
		else
			echo -e "${color_red}datto utils install failed.${color_off}"	
		fi

	else
		echo ""
		echo -e "${color_blue}Already installed rpm packages:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $EXIST_datto ]"${color_off}"
		echo ""
	fi
fi
}

suse_datto_install() {
print_head

Chk_dracut_ver

echo -e "${color_blue}[ datto driver install ]${color_off}"
req_datto_rpm
if [ ${req_datto} -eq 0 ]; then
	if [ -n "$MISSING_datto" ]; then
		echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $MISSING_datto ]"${color_off}"
		echo ""

		if [ -e $dir/packages -a -e $dir/packages/datto ]; then
			suse_datto_rpm
		else
			echo "datto package is not exist."
			exit
		fi

		modinfo dattobd > /dev/null 2>&1
		if [ $? -eq 0 ]; then
			echo -e "${color_blue}dattobd Driver install successful.${color_off}"	
		else
			echo -e "${color_red}dattobd Driver install failed.${color_off}"
		fi
		
		mod_cnf+=(`find /etc/modprobe.d/ /lib/modprobe.d/ -name "10-*" -or -name "20-*" 2>/dev/null`)
		if [ ${#mod_cnf[@]} -ne 0 ]; then
			for cnf in "${mod_cnf[@]}"
			do
				mod_val=`cat $cnf | grep -E "^allow_unsupported_modules" | awk -F " " '{print $2}'`
				if [ "$mod_val" -eq 0 ]; then
					sed -i '/^#/!s/0/1/gi' $cnf
					if [ $? -eq 0 ]; then
						echo "`date '+%m/%d/%Y %H:%M:%S.%3N'` ** Set allow_unsupported_modules 1."
						ret=0
					else
						ret=1
					fi
				elif [ "$mod_val" -eq 1 ]; then
					echo "`date '+%m/%d/%Y %H:%M:%S.%3N'` ** Already set allow_unsupported_modules 1."
					ret=0
				fi
			done
		else
			modprobe dattobd
		fi
		if [ $ret -eq 0 ]; then
			modprobe dattobd
		else
			echo "`date '+%m/%d/%Y %H:%M:%S.%3N'` ** Failed to set allow_unsupported_modules."
		fi
		
		sleep 3
		echo ""
		echo "[ /proc/datto-info ]"
		cat /proc/datto-info

		chk_dbdctl=`dbdctl | grep "dbdctl help"| wc -l`
		if [ $chk_dbdctl -eq 1 ]; then
			echo -e "${color_blue}datto utils install successful.${color_off}" 
		else
			echo -e "${color_red}datto utils install failed.${color_off}"	
		fi

	else
		echo ""
		echo -e "${color_blue}Already installed rpm packages:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $EXIST_datto ]"${color_off}"
		echo ""
	fi
fi
}

Debian_datto_install() {
print_head

Chk_dracut_ver

echo -e "${color_blue}[ datto driver install ]${color_off}"
req_datto_dpkg
if [ ${req_datto} -eq 0 ]; then
	if [ -n "$MISSING_datto" ]; then
		echo -e "${color_blue}The following dpkg packages are not installed:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $MISSING_datto ]"${color_off}"
		echo ""

		if [ -e $dir/packages -a -e $dir/packages/datto ]; then
			Debian_datto_dpkg
		else	
			echo "datto package is not exist."
			exit
		fi

		modinfo dattobd > /dev/null 2>&1
		if [ $? -eq 0 ]; then
			echo -e "${color_blue}dattobd Driver install successful.${color_off}"	
		else
			echo -e "${color_red}dattobd Driver install failed.${color_off}"
		fi
		
		datto_path=`modinfo -F filename dattobd`
		insmod $datto_path > /dev/null 2>&1 || true

		echo ""
		echo "[ /proc/datto-info ]"
		cat /proc/datto-info

		chk_dbdctl=`dbdctl | grep "dbdctl help"| wc -l`
		if [ $chk_dbdctl -eq 1 ]; then
			echo -e "${color_blue}datto utils install successful.${color_off}" 
		else
			echo -e "${color_red}datto utils install failed.${color_off}"	
		fi

	else
		echo ""
		echo -e "${color_blue}Already installed dpkg packages:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $EXIST_datto ]"${color_off}"
		echo ""
	fi
fi
}

amazon_datto_install() {
print_head

Chk_dracut_ver

echo -e "${color_blue}[ datto driver install ]${color_off}"
req_datto_rpm
if [ ${req_datto} -eq 0 ]; then
	if [ -n "$MISSING_datto" ]; then
		echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $MISSING_datto ]"${color_off}"
		echo ""

		if [ -e $dir/packages -a -e $dir/packages/datto ]; then
			amazon_datto_rpm
		else
			echo "datto package is not exist."
			exit
		fi

		modinfo dattobd > /dev/null 2>&1
		if [ $? -eq 0 ]; then
			echo -e "${color_blue}dattobd Driver install successful.${color_off}"	
		else
			echo -e "${color_red}dattobd Driver install failed.${color_off}"
		fi
		
		datto_path=`modinfo -F filename dattobd`
		insmod $datto_path > /dev/null 2>&1 || true

		echo ""
		echo "[ /proc/datto-info ]"
		cat /proc/datto-info

		chk_dbdctl=`dbdctl | grep "dbdctl help"| wc -l`
		if [ $chk_dbdctl -eq 1 ]; then
			echo -e "${color_blue}datto utils install successful.${color_off}" 
		else
			echo -e "${color_red}datto utils install failed.${color_off}"	
		fi

	else
		echo ""
		echo -e "${color_blue}Already installed rpm packages:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $EXIST_datto ]"${color_off}"
		echo ""
	fi
fi
}


AIPLinux_install()
{
print_head

echo -e "${color_blue}[ AIPLinux install ]${color_off}"
if [ "$OSDIST" = "Debian" ] || [ "$OSDIST" = "Ubuntu" ]; then
	req_AIP_dpkg
else
	req_AIP_rpm
fi
if [ ${req_AIP} -eq 0 ]; then
	if [ -f $dir/AIPLinuxInstaller ]; then
		export LANG=$saved_lang
		exec > $tty
		bash AIPLinuxInstaller
		export LANG=C
		exec > >(tee -a ${LOG}) 2>&1
	else
		echo "AIPLinuxInstaller does not exist."
		echo -n "Input the AIPLinuxInstaller path [e.g.: /tmp ] : "
		read txt_path
		if [ -f $txt_path/AIPLinuxInstaller ]; then 
			export LANG=$saved_lang
			exec > $tty
			bash $txt_path/AIPLinuxInstaller
			export LANG=C
			exec > >(tee -a ${LOG}) 2>&1
		else

			echo "AIPLinuxInstaller does not exist."
			exit 1
		fi
	fi
fi
}

NoNeed(){
echo $NotNeed
	for rpm in $NotNeed; do
		echo "[ rpm -e $rpm ]"
		rpm -e $rpm
	done
}

datto_rpm_remove() {

	ask_YN "Do you want to uninstall datto driver and rpm?" "Y"
	if [ "$RC_ask_YN" = "Y" ]; then
		for rpm in $EXIST_datto; do
			echo "[ rpm -e $rpm ]"
			rpm -e $rpm
		done
		#$inst_com remove $EXIST_datto

		modinfo dattobd > /dev/null 2>&1
		if [ $? -eq 1 ]; then
			echo -e "${color_blue}dattobd Driver uninstall successful.${color_off}"	
		else
			echo -e "${color_red}dattobd Driver uninstall failed.${color_off}"
		fi

		dbdctl help > /dev/null 2>&1
		if [ $? -eq 127 ]; then
			echo -e "${color_blue}datto utils uninstall successful.${color_off}" 
		else
			echo -e "${color_red}datto utils uninstall failed.${color_off}"	
		fi
	fi
}

Debian_datto_rpm_remove() {

	ask_YN "Do you want to uninstall datto driver and rpm?" "Y"
	if [ "$RC_ask_YN" = "Y" ]; then
		for dpkg in $EXIST_datto; do
			echo "[ apt remove $dpkg ]"
			apt remove -y $dpkg
		done
		#$inst_com remove $EXIST_datto

		modinfo dattobd > /dev/null 2>&1
		if [ $? -eq 1 ]; then
			echo -e "${color_blue}dattobd Driver uninstall successful.${color_off}"	
		else
			echo -e "${color_red}dattobd Driver uninstall failed.${color_off}"
		fi

		dbdctl help > /dev/null 2>&1
		if [ $? -eq 127 ]; then
			echo -e "${color_blue}datto utils uninstall successful.${color_off}" 
		else
			echo -e "${color_red}datto utils uninstall failed.${color_off}"	
		fi
	fi
}

datto_remove() {
print_head

echo -e "${color_blue}[ datto driver uninstall ]${color_off}"
echo ""

if [ -n "$EXIST_datto" ]; then
	echo -e "${color_blue}The following rpm packages are installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST_datto ]"${color_off}"
	echo ""
	if [ "$OSDIST" = "Debian" ] || [ "$OSDIST" = "Ubuntu" ]; then
		Debian_datto_rpm_remove
	else
		NoNeed		
		datto_rpm_remove
	fi
else
	echo -e "${color_blue}The datto rpm packages are not exists.${color_off}"
	echo ""
	NoNeed
fi

}

pre_aas(){

chkGUI=`yum grouplist installed | grep -e "GUI" -e "KDE" -e "Xface" | wc -l`
chkSession=`find /usr/share/xsessions/ /usr/share/wayland-sessions/ -name "*.desktop" | wc -l`

chkDocker=`rpm -qa | grep -e docker-ce -e docker-ee | wc -l`
if [ $chkDocker -eq 0 ]; then
	if [ $chkGUI -ne 0 ] && [ $chkSession -ge 1 ]; then
		echo "GUI environment"
		rhel_list="podman podman-docker google-chrome-stable"
		aas_rpm=(podman podman-docker google-chrome-stable)
	else
		echo "Non GUI environment"
		rhel_list="podman podman-docker"
		aas_rpm=(podman podman-docker)
	fi
else
	if [ $chkGUI -ne 0 ] && [ $chkSession -ge 1 ]; then
		echo "GUI environment"
		rhel_list="google-chrome-stable"
		aas_rpm=(google-chrome-stable)
		dockerCE=1
	else
		echo "Non GUI environment"
		dockerCE=1
	fi
fi
	dockerCE=0
	unset EXIST_aas
	unset MISSING_aas
	unset arryE
	unset arryM

	if [ "$OSDIST" = "openSUSE" ] || [ "$OSDIST" = "SLES" ] || [ "$OSDIST" = "SLED" ]; then
		echo "["$OSDIST"] version["$OSVER"] does not support."
		echo ""
		exit
	elif [ "$OSDIST" = "Debian" ] || [ "$OSDIST" = "Ubuntu" ]; then
		echo "["$OSDIST"] version["$OSVER"] does not support."
		echo ""
		exit
	else
		if [ $OSVER -le 7 ]; then
			echo "["$OSDIST"] version["$OSVER"] does not support."
			echo ""
			exit
		else
			for aas_e in $rhel_list; do
				if rpm -q $aas_e &>/dev/null; then
					if [[ $aas_e = *.x86_64 || *.i686 ]]; then
						EXIST_aas="$aas_e $EXIST_aas"
						arryE+=($aas_e)
					fi
				fi
			done
			for aas_m in $rhel_list; do
				if ! rpm -q $aas_m &>/dev/null; then
					MISSING_aas="$aas_m $MISSING_aas"
					arryM+=($aas_m)
				fi
			done
		fi
	fi

if [ -n "$EXIST_aas" ]; then
	BK_IFS=$IFS
	IFS=$'\n'
	if [ ! -f /etc/.aas ]; then
		echo "${arryE[*]}" >/etc/.aas
	fi
	IFS=$BK_IFS
fi
}

exist_package(){
unset arry2
unset arry3
unset uniq_arry
unset not_uniq_arry

BK_IFS=$IFS
IFS=$'\n'
if [ -f /etc/.aas ]; then
	while read line; do
		arry2+=($line)
	done < <(cat /etc/.aas)
fi
arry3=("${arry2[@]}" "${aas_rpm[@]}")
uniq_arry=$(printf "%s\n" "${arry3[@]}" | sort | uniq -u)
not_uniq_arry=$(printf "%s\n" "${arry3[@]}" | sort | uniq -d)
IFS=$BK_IFS
}

del_exist_package(){
unset arry2
unset arry3
unset uniq_arry
unset not_uniq_arry

BK_IFS=$IFS
IFS=$'\n'
if [ -f /etc/.aas ]; then
	while read line; do
		arry2+=($line)
	done < <(cat /etc/.aas)
fi
arry3=("${arry2[@]}" "${aas_rpm[@]}")
uniq_arry=$(printf "%s\n" "${arry3[@]}" | sort | uniq -u)
not_uniq_arry=$(printf "%s\n" "${arry3[@]}" | sort | uniq -d)
IFS=$BK_IFS
}

google_repo() {
	echo [google-chrome] >> /etc/yum.repos.d/google.chrome.repo
	echo name=google-chrome >> /etc/yum.repos.d/google.chrome.repo
	echo baseurl=http://dl.google.com/linux/chrome/rpm/stable/x86_64 >> /etc/yum.repos.d/google.chrome.repo
	echo enable=1 >> /etc/yum.repos.d/google.chrome.repo
	echo gpgcheck=0 >> /etc/yum.repos.d/google.chrome.repo
}

aas_Unit(){

if [ $dockerCE -eq 1 ]; then
	docker update --restart=always actiphy_aas
else
	docker generate systemd --new --name actiphy_aas > /etc/systemd/system/actiphy_aas.service
	if [ $? -eq 0 ]; then
		systemctl enable actiphy_aas.service
		systemctl start actiphy_aas.service
	fi
fi
}

rhel_docker(){
if [ ! -f /tmp/AAS_docker.tar.gz ]; then
	aasTar="http://product-binaries.s3.actiphy.com/docker/AAS_docker.tar.gz"
	echo "Downloading Docker image...."
	wget -nv -P /tmp/ $aasTar
	if [ $? -ne 0 ]; then
		echo -e "${color_red}Failed to download AAS Docker image.${color_off}"	
		echo -e "${color_red}Please download AAS Docker image.${color_off}"
		echo -e "${color_red}$aasTar${color_off}"
		echo -e "${color_red}Please save the AAS Docker image to /tmp .${color_off}"
	else
		echo "Loading Docker image...."
		if [ $dockerCE -eq 1 ]; then
			if systemctl status docker | grep running 2>&1 ; then
				systemctl start docker 2>&1
			fi
		fi
		docker load -i "/tmp/AAS_docker.tar.gz"
		im=`docker load -i "/tmp/AAS_docker.tar.gz" | grep "Loaded image" | awk '{print $3}' | sed -e 's/[a-z].*\///'`
		echo $im
		docker run -d --name actiphy_aas -p "443:443" -p "55236:55236" -p "55237:55237" -p "55238:55238/udp" -p "55239:55239/udp" -v actiphy-aas-log:/home/aas/logs $im
		if [ $? -eq 0 ]; then
			echo -e "${color_blue}AAS install successful.${color_off}"	
		else
			echo -e "${color_red}AAS install failed.${color_off}"
		fi
	fi
else
	echo "Loading Docker image...."
	docker load -i "/tmp/AAS_docker.tar.gz"
	im=`docker load -i "/tmp/AAS_docker.tar.gz" | grep "Loaded image" | awk '{print $3}' | sed -e 's/[a-z].*\///'`
	echo $im
	docker run -d --name actiphy_aas -p "443:443" -p "55236:55236" -p "55237:55237" -p "55238:55238/udp" -p "55239:55239/udp" -v actiphy-aas-log:/home/aas/logs $im
	if [ $? -eq 0 ]; then
		echo -e "${color_blue}AAS install successful.${color_off}"	
	else
		echo -e "${color_red}AAS install failed.${color_off}"
	fi
fi
}

chrome_postset(){
instPath=/opt/google/chrome
chrome_ini=$instPath/google-chrome
if [ -d $instPath ]; then
	if [ -f $chrome_ini ]; then
		chkChrome=`cat $chrome_ini | grep "no-sandbox" | wc -l`
		if [ $chkChrome -eq 0 ]; then
			sed -i 's/"$@"/"$@" --no-sandbox/' $chrome_ini
		fi
	fi
fi
}

install_docker(){

pre_aas
google_repo
exist_package

if [ -n "$MISSING_aas" ]; then
	echo -e "${color_blue}The following packages are not installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $MISSING_aas ]"${color_off}"
	echo ""

#runc1.0.0-70.rc92 bug
#Emulate Docker CLI using podman. Create /etc/containers/nodocker to quiet msg.
#Error: container_linux.go:370: starting container process caused: error adding seccomp filter rule for syscall bdflush: permission denied: OCI permission denied

	if [ $net_ok -eq 0 ]; then 	#つながる場合
		echo "=== Internet connection OK."
		existRunc=`rpm -qa  | grep runc | wc -l`	
		if [ $existRunc -eq 1 ]; then
			chkRunc=`rpm -qi runc | grep Version | awk '{print $NF}' | sed -r 's/\.//g'`
			chkConmon=`rpm -qi conmon | grep Version | awk '{print $NF}' | sed -r 's/\.//g'`
			if [ -n "$chkRunc" ] && [ $chkRunc -lt 102 ]; then
				echo "runc ver "$chkRunc
				echo "Update runc package."
				$inst_com update -q runc
				if [ $? -eq 1 ]; then
						echo -e "${color_red}Failed to update runc.${color_off}"
						echo -e "${color_red}Please get latest version runc in another way.${color_off}"
						exit 1
					fi
				fi
			if [ -n "$chkConmon" ] && [ $chkConmon -lt 2024 ]; then
				echo "conmon ver "$chkConmon
				echo "Update conmon package."
				$inst_com update -q conmon
				if [ $? -eq 1 ]; then
					echo -e "${color_red}Failed to update conmon.${color_off}"
					echo -e "${color_red}Please get latest version conmon in another way.${color_off}"
					exit 1
				fi
			elif [ -z "$chkConmon" ]; then
				echo "Not exist conmon package."
				echo "Install conmon package"
				$inst_com install -q conmon
				if [ $? -eq 1 ]; then
					echo -e "${color_red}Failed to install conmon.${color_off}"
					echo -e "${color_red}Please get latest version conmon in another way.${color_off}"
					exit 1
				fi
			fi
		else	
			echo "Not exist runc package."
			echo "Install runc package"
			$inst_com install -q runc
			if [ $? -eq 1 ]; then
				echo -e "${color_red}Failed to install conmon.${color_off}"
				echo -e "${color_red}Please get latest version conmon in another way.${color_off}"
				exit 1
			fi
		fi
		$inst_com install $MISSING_aas
		if [ $? -eq 1 ]; then
			echo -e "${color_red}yum install is failed.${color_off}"
			echo -e "${color_red}Please get "$MISSING_aas" in another way.${color_off}"
			echo -e "${color_red}Please install "$MISSING_aas".${color_off}"
			echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
			exit 1
		else
			rhel_docker
		fi
	fi
else
	echo ""
	echo -e "${color_blue}Already installed rpm packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST_aas ]"${color_off}"
	echo ""
	rhel_docker
fi
rm -rf /tmp/AAS_docker*
rm -rf /var/tmp/docker-tar*
}

remove_container(){

unset imageID_arry
unset dockerID_arry
unset AAS

imageID_arry+=(`docker images 2>&1 | grep aas | awk '{print $3}'`)
dockerID_arry+=(`docker ps -a --format "{{.ID}}  {{.Image}}  {{.Labels}}" | grep -E "actiphy_aas" | awk '{print $1}'`)

for aasID in ${imageID_arry[@]}; do
	chkAAS=`docker inspect $aasID | grep -E "aas\:[0-9]\.[0-9]\.[0-9].[0-9]*" | wc -l`
	if [ $chkAAS -ne 0 ]; then
		AAS+=($aasID)
	fi
done 

if [ -n "$AAS" ]; then
	for dockerID in ${dockerID_arry[@]}; do
		docker stop $dockerID
	done
	for imageID in ${AAS[@]}; do
		docker rmi $imageID -f
	done
	
	AASImage=`docker images 2>&1 | grep aas | wc -l`
	AASPS=`docker ps 2>&1 | grep actiphy | awk '{print $1}'`
	if [ $AASImage -eq 0 ] && [ -z "$AASPS" ]; then
		echo -e "${color_blue}AAS remove successful.${color_off}" 
	else
		echo -e "${color_red}AAS remove failed.${color_off}"	
	fi
fi
}

aas() {
print_head

echo -e "${color_blue}[ AAS install ]${color_off}"
echo ""

AASImage=`docker images 2>&1 | grep aas | wc -l`
AASPS=`docker ps 2>&1 | grep actiphy | awk '{print $1}'`
if [ $AASImage -ge 1 ] && [ -n "$AASPS" ]; then
	echo -e "${color_blue}Already exist AAS.${color_off}"
	ask_YN "Would you like to update AAS image?" "Y"
	if [ "$RC_ask_YN" = "N" ]; then
		:
	else
		remove_container
		install_docker
		aas_Unit
	fi
else
	install_docker
	aas_Unit
fi
chrome_postset
}

aas_remove(){
print_head

echo -e "${color_blue}[ AAS uninstall ]${color_off}"
echo ""

pre_aas
del_exist_package
	#echo "aas_e " ${arryE[@]}
	#echo "aas_m " ${arryM[@]}
	#echo "uniq rpm "${uniq_arry[@]}
	#echo "not uniq rpm " ${not_uniq_arry[@]}

	arry4=("${arryM[@]}" "${uniq_arry[@]}")
	only=$(printf "%s\n" "${arry4[@]}" | sort | uniq -u)
	both=$(printf "%s\n" "${arry4[@]}" | sort | uniq -d)

	#echo "only " ${only[@]}
	#echo "both " ${both[@]}

if [ -n "$EXIST_aas" ]; then
	echo -e "${color_blue}The following rpm packages are installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST_aas ]"${color_off}"
	echo ""
	dockerIM=`docker images 2>&1 | grep aas | awk '{print $1}'`
	if [ -n "$dockerIM" ]; then
		remove_container
		if [ ! -f /etc/.aas ]; then
			echo $inst_com autoremove $EXIST_aas
			$inst_com autoremove $EXIST_aas
		else
			if [ -z "${uniq_arry[@]}" ]; then
				echo Pre-installed $EXIST_aas.
				echo Does not remove $EXIST_aas.
			else
				if [ -z "${only[@]}" ]; then
					echo Pre-installed $EXIST_aas.
					echo Does not remove $EXIST_aas.
					echo -e "${color_magenta}Nothing to do."
				else
					echo -e "${color_magenta}Pre-installed "${not_uniq_arry[@]}".${color_off}"
					echo -e "${color_magenta}Does not remove "${not_uniq_arry[@]}".${color_off}"			
					echo $inst_com autoremove ${uniq_arry[@]}
					$inst_com autoremove ${uniq_arry[@]}
				fi
			fi
		fi
		if [ $? -eq 0 ]; then
			echo -e "${color_blue}AAS package remove successful.${color_off}" 
			rm -rf /tmp/AAS_docker*
		else
			echo -e "${color_red}AAS package remove failed.${color_off}" 
		fi
	else
		if [ ! -f /etc/.aas ]; then
			echo $inst_com autoremove $EXIST_aas
			$inst_com autoremove $EXIST_aas
		else
			if [ -z "${uniq_arry[@]}" ]; then
				echo Pre-installed $EXIST_aas.
				echo Does not remove $EXIST_aas.
			else
			if [ -z "${only[@]}" ]; then
					echo Pre-installed $EXIST_aas.
					echo Does not remove $EXIST_aas.
					echo -e "${color_magenta} Nothing to do."
				fi
			fi
		fi
		if [ $? -eq 0 ]; then
			echo -e "${color_blue}AAS package remove successful.${color_off}" 
			rm -rf /tmp/AAS_docker*
		else
			echo -e "${color_red}AAS package remove failed.${color_off}" 
		fi
	fi
	google_remove
else
	echo -e "${color_blue}AAS packages are not exists.${color_off}"
	echo ""
	dockerID=`docker ps 2>&1 | grep actiphy | awk '{print $1}'`
	if [ -n "$dockerID" ]; then
		remove_container
		if [ ! -f /etc/.aas ]; then
			if [ -z $EXIST_aas ]; then
				echo "Nothing to do."
			else
				$inst_com autoremove $EXIST_aas
			fi
		fi
		if [ $? -eq 0 ]; then
			echo -e "${color_blue}AAS package remove successful.${color_off}" 
			rm -rf /tmp/AAS_docker*
		else
			echo -e "${color_red}AAS package remove failed.${color_off}" 
		fi
	else
		if [ ! -f /etc/.aas ]; then
			if [ -z $EXIST_aas ]; then
				echo "Nothing to do."
			else
				$inst_com autoremove $EXIST_aas
			fi
		fi
		if [ $? -eq 0 ]; then
			echo -e "${color_blue}AAS package remove successful.${color_off}" 
			rm -rf /tmp/AAS_docker*
		else
			echo -e "${color_red}AAS package remove failed.${color_off}" 
		fi
	fi
	google_remove
fi

rm -rf /var/tmp/docker-tar*
}

#######################################################################
# Main
#######################################################################

show_help()
{
	print_head

	echo "Options:"
	printf "\tAgent\t\t\t: Install required packages for Agent.\n"
	printf "\tGUI\t\t\t: Install required packages for GUI.\n"
	printf "\t[1]-[2]All\t\t: Install required packages for Agent and GUI.\n"
	printf "\tKernel-devel\t\t: Install required Kernel-devel.\n"
	printf "\tInstall datto driver\t: Install datto driver.\n"
	printf "\tInstall AIPLinux\t: Install AIPLinux.\n"
	printf "\tUninstall datto driver\t: Uninstall datto driver.\n"
	printf "\tInstall AAS \t\t: Install AAS for docker.\n"
	printf "\tUninstall AAS \t\t: Uninstall AAS for docker.\n"
	printf "\thelp\t\t\t: Print this help.\n"
	printf "\tExit\t\t\t: Exit.\n"
	echo ""
}

Check_sb

case "$OSVER" in
2)
	List_Agent="cifs-utils make.x86_64 gcc.x86_64 zip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 rsync.x86_64 samba-client.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64"

	List_GUI="expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64 systemd-libs.x86_64"

	List_All="cifs-utils make.x86_64 gcc.x86_64 zip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 rsync.x86_64 samba-client.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64 expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64 systemd-libs.x86_64"
	;;
6)
	List_Agent="cifs-utils make.x86_64 gcc.x86_64 zip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 rsync.x86_64 libsmbclient.x86_64 samba-client.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64"
	
	List_GUI="expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64"

	List_All="cifs-utils make.x86_64 gcc.x86_64 zip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 rsync.x86_64 libsmbclient.x86_64 samba-client.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64 expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64"
	;;
7)
	List_Agent="cifs-utils make.x86_64 gcc.x86_64 zip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 rsync.x86_64 samba-client.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64"

	List_GUI="expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64 systemd-libs.x86_64"

	List_All="cifs-utils make.x86_64 gcc.x86_64 zip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 rsync.x86_64 samba-client.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64 expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64 systemd-libs.x86_64"
	;;
8)
	List_Agent="cifs-utils make.x86_64 gcc.x86_64 perl.x86_64 zip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 rsync.x86_64 samba-client.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64 elfutils-libelf-devel.x86_64 libnsl.x86_64"

	List_GUI="expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64 systemd-libs.x86_64 zlib-devel.x86_64"

	List_All="cifs-utils make.x86_64 gcc.x86_64 perl.x86_64 zip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 rsync.x86_64 samba-client.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64 expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64 systemd-libs.x86_64 elfutils-libelf-devel.x86_64 zlib-devel.x86_64 libnsl.x86_64"
	;;
9)
	List_Agent="cifs-utils make.x86_64 gcc.x86_64 perl.x86_64 zip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 rsync.x86_64 samba-client.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64 elfutils-libelf-devel.x86_64 libnsl.x86_64"

	List_GUI="expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64 systemd-libs.x86_64 zlib-devel.x86_64"

	List_All="cifs-utils make.x86_64 gcc.x86_64 perl.x86_64 zip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 rsync.x86_64 samba-client.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64 expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64 systemd-libs.x86_64 elfutils-libelf-devel.x86_64 zlib-devel.x86_64 libnsl.x86_64"
	;;
94)
	List_Agent="cifs-utils libxml2-utils make gcc perl zip unzip vim-common wget fuse rsync smbclient iptables-persistent netfilter-persistent libc6 libgcc-6-dev libstdc++6 libuuid-perl libnss3 zlib1g"

	List_GUI="iptables-persistent netfilter-persistent libexpat1 fontconfig libfreetype6 libglib2.0-0 libice6 libsmartcols1 libx11-data libxau6 libxext6 libxrender1 libxcb1 mesa-utils systemd libelf1 zlib1g-dev libnss-systemd"

	List_All="cifs-utils libxml2-utils make gcc perl zip unzip vim-common wget fuse rsync smbclient iptables-persistent netfilter-persistent libc6 libgcc-6-dev libstdc++6 libuuid-perl libnss3 zlib1g libexpat1 fontconfig libfreetype6 libglib2.0-0 libice6 libsmartcols1 libx11-data libxau6 libxext6 libxrender1 libxcb1 mesa-utils systemd libelf1 zlib1g-dev libnss-systemd"
	;;
10)
	List_Agent="cifs-utils libxml2-utils make gcc perl zip unzip vim-common wget fuse rsync smbclient iptables-persistent netfilter-persistent libc6 libgcc-8-dev libstdc++6 libuuid-perl libnss3 zlib1g"

	List_GUI="iptables-persistent netfilter-persistent expat fontconfig libfreetype6 libglib2.0-0 libice6 libsmartcols1 libx11-data libxau6 libxext6 libxrender1 libxcb1 mesa-utils systemd libelf1 zlib1g-dev libnss-systemd"

	List_All="cifs-utils libxml2-utils make gcc perl zip unzip vim-common wget fuse rsync smbclient iptables-persistent netfilter-persistent libc6 libgcc-8-dev libstdc++6 libuuid-perl libnss3 zlib1g expat fontconfig libfreetype6 libglib2.0-0 libice6 libsmartcols1 libx11-data libxau6 libxext6 libxrender1 libxcb1 mesa-utils systemd libelf1 zlib1g-dev libnss-systemd"
	;;
	
11)
	List_Agent="cifs-utils libxml2-utils make gcc perl zip unzip vim-common wget fuse rsync smbclient iptables-persistent netfilter-persistent libc6 libgcc-9-dev libstdc++6 libuuid-perl libnss3 zlib1g"

	List_GUI="iptables-persistent netfilter-persistent expat fontconfig libfreetype6 libglib2.0-0 libice6 libsmartcols1 libx11-data libxau6 libxext6 libxrender1 libxcb1 mesa-utils systemd libelf1 zlib1g-dev libnss-systemd"

	List_All="cifs-utils libxml2-utils make gcc perl zip unzip vim-common wget fuse rsync smbclient iptables-persistent netfilter-persistent libc6 libgcc-9-dev libstdc++6 libuuid-perl libnss3 zlib1g expat fontconfig libfreetype6 libglib2.0-0 libice6 libsmartcols1 libx11-data libxau6 libxext6 libxrender1 libxcb1 mesa-utils systemd libelf1 zlib1g-dev libnss-systemd"
	;;

15)
	List_Agent="cifs-utils make.x86_64 gcc.x86_64 perl.x86_64 zip.x86_64 unzip.x86_64 vim.x86_64 wget.x86_64 fuse.x86_64 rsync.x86_64 samba-client.x86_64 glibc.x86_64 libgcc_s1.x86_64 libstdc++6.x86_64 libuuid1.x86_64 libfreebl3.x86_64 libz1.x86_64 elfutils.x86_64 libnsl2.x86_64"

	List_GUI="expat.x86_64 fontconfig.x86_64 libfreetype6.x86_64 libglib-2_0-0.x86_64 libICE6.x86_64 libSM6.x86_64 libX11-6.x86_64 libXau6.x86_64 libXext6.x86_64 libXrender1.x86_64 libxcb1.x86_64 Mesa-libGL1.x86_64 libsystemd0.x86_64 zlib-devel.x86_64"

	List_All="cifs-utils make.x86_64 gcc.x86_64 perl.x86_64 zip.x86_64 unzip.x86_64 vim.x86_64 wget.x86_64 fuse.x86_64 rsync.x86_64 samba-client.x86_64 libglib-2_0-0.x86_64 glibc.x86_64 libgcc_s1.x86_64 libstdc++6.x86_64 libuuid1.x86_64 libfreebl3.x86_64 libz1.x86_64 elfutils.x86_64 libnsl2.x86_64 expat.x86_64 fontconfig.x86_64 libfreetype6.x86_64 libICE6.x86_64 libSM6.x86_64 libX11-6.x86_64 libXau6.x86_64 libXext6.x86_64 libXrender1.x86_64 libxcb1.x86_64 Mesa-libGL1.x86_64 libsystemd0.x86_64 zlib-devel.x86_64"
	;;

18)
	List_Agent="cifs-utils libxml2-utils make gcc perl zip unzip vim-common wget fuse rsync smbclient iptables-persistent netfilter-persistent libc6 libgcc-8-dev libgcc1 libstdc++6 libuuid-perl libnss3 zlib1g"

	List_GUI="iptables-persistent netfilter-persistent expat fontconfig libfreetype6 libglib2.0-0 libice6 libsmartcols1 libx11-data libxau6 libxext6 libxrender1 libxcb1 mesa-utils systemd libelf1 zlib1g-dev libnss-systemd"

	List_All="cifs-utils libxml2-utils make gcc perl zip unzip vim-common wget fuse rsync smbclient iptables-persistent netfilter-persistent libc6 libgcc-8-dev  libgcc1 libstdc++6 libuuid-perl libnss3 zlib1g expat fontconfig libfreetype6 libglib2.0-0 libice6 libsmartcols1 libx11-data libxau6 libxext6 libxrender1 libxcb1 mesa-utils systemd libelf1 zlib1g-dev libnss-systemd"
	;;
20)
	List_Agent="cifs-utils libxml2-utils make gcc perl zip unzip vim-common wget fuse rsync smbclient iptables-persistent netfilter-persistent libc6 libgcc-9-dev libgcc-s1 libstdc++6 libuuid-perl libnss3 zlib1g"

	List_GUI="iptables-persistent netfilter-persistent expat fontconfig libfreetype6 libglib2.0-0 libice6 libsmartcols1 libx11-data libxau6 libxext6 libxrender1 libxcb1 mesa-utils systemd libelf1 zlib1g-dev libnss-systemd"

	List_All="cifs-utils libxml2-utils make gcc perl zip unzip vim-common wget fuse rsync smbclient iptables-persistent netfilter-persistent libc6 libgcc-9-dev libgcc-s1 libstdc++6 libuuid-perl libnss3 zlib1g expat fontconfig libfreetype6 libglib2.0-0 libice6 libsmartcols1 libx11-data libxau6 libxext6 libxrender1 libxcb1 mesa-utils systemd libelf1 zlib1g-dev libnss-systemd"
	;;
22)
	List_Agent="cifs-utils libxml2-utils make gcc perl zip unzip vim-common wget fuse rsync smbclient iptables-persistent netfilter-persistent libc6 libgcc-9-dev libgcc-s1 libstdc++6 libuuid-perl libnss3 zlib1g"

	List_GUI="iptables-persistent netfilter-persistent expat fontconfig libfreetype6 libglib2.0-0 libice6 libsmartcols1 libx11-data libxau6 libxext6 libxrender1 libxcb1 mesa-utils systemd libelf1 zlib1g-dev libnss-systemd"

	List_All="cifs-utils libxml2-utils make gcc perl zip unzip vim-common wget fuse rsync smbclient iptables-persistent netfilter-persistent libc6 libgcc-9-dev libgcc-s1 libstdc++6 libuuid-perl libnss3 zlib1g expat fontconfig libfreetype6 libglib2.0-0 libice6 libsmartcols1 libx11-data libxau6 libxext6 libxrender1 libxcb1 mesa-utils systemd libelf1 zlib1g-dev libnss-systemd"
	;;
esac

if [ -f /etc/yum.repos.d/DVD.repo ]; then
	rm /etc/yum.repos.d/DVD.repo
fi

Check_internet
Check_subsc

ohNo(){
if [ "$OSVER" = "7" ] && [ "$OSDIST" = "Asianux" ]; then
	if [ $min_ver -lt 3 ]; then
		echo -e "${color_red}This OS version SP$min_ver, it has been confirmed that installing a package over the insternet causes [Oh no!] symptom.${color_off}"
		echo -e "${color_red}yum command has been aborted over the Internet.${color_off}"
		echo -e "${color_blue}Please try SP4 or later.${color_off}"
		echo -e "${color_blue}Or please run the script without internet connection.${color_off}"
		exit
	fi
elif [ "$OSVER" = "7" ]; then
var=$(awk 'BEGIN{print "'7.5'"<"'$min_ver'" }')
	if [ "$var" -ne 1 ] && [ $net_ok -eq 0 ]; then
		echo -e "${color_red}This OS version [$min_ver], it has been confirmed that installing a package over the insternet causes [Oh no!] symptom.${color_off}"
		echo -e "${color_red}yum command has been aborted over the Internet.${color_off}"
		echo -e "${color_blue}Please try Ver7.5 or later.${color_off}"
		echo -e "${color_blue}Or please run the script without internet connection.${color_off}"
		exit
	fi
fi
}
if [ "$OSDIST" = "RHEL" ] || [ "$OSDIST" = "CentOS" ] || [ "$OSDIST" = "Asianux" ] || [ "$OSDIST" = "AlmaLinux" ] || [ "$OSDIST" = "MIRACLE" ] || [ "$OSDIST" = "Rocky" ]; then
	select_options
elif [ "$OSDIST" = "Oracle" ]; then
	oracle_select_options
elif [ "$OSDIST" = "openSUSE" ] || [ "$OSDIST" = "SLED" ] || [ "$OSDIST" = "SLES" ]; then
	suse_select_options
elif [ "$OSDIST" = "Debian" ]; then
	set_sudo
	pre_debian_repo
	Debian_select_options
elif [ "$OSDIST" = "Ubuntu" ]; then
	pre_debian_repo
	Debian_select_options
elif [ "$OSDIST" = "Amazon" ]; then
	Amazon_select_options
fi

vault_remove
DVD_remove
ol_remove
echo "** `date '+%Y-%m-%d %H:%M:%S'` -END"
exit 0
